# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ast
from tempest.lib.cli import output_parser
import testtools

from manilaclient import api_versions
from manilaclient import config

CONF = config.CONF


def multi_line_row_table(output_lines, group_by_column_index=0):
    parsed_table = output_parser.table(output_lines)

    rows = parsed_table['values']
    row_index = 0

    def get_column_index(column_name, headers, default):
        return next(
            (i for i, h in enumerate(headers) if h.lower() == column_name),
            default
        )

    if group_by_column_index is None:
        group_by_column_index = get_column_index(
            'id', parsed_table['headers'], 0)

    def is_embedded_table(parsed_rows):
        def is_table_border(t):
            return str(t).startswith('+')

        return (isinstance(parsed_rows, list)
                and len(parsed_rows) > 3
                and is_table_border(parsed_rows[0])
                and is_table_border(parsed_rows[-1]))

    def merge_cells(master_cell, value_cell):
        if value_cell:
            if not isinstance(master_cell, list):
                master_cell = [master_cell]
            master_cell.append(value_cell)

        if is_embedded_table(master_cell):
            return multi_line_row_table('\n'.join(master_cell), None)

        return master_cell

    def is_empty_row(row):
        empty_cells = 0
        for cell in row:
            if cell == '':
                empty_cells += 1
        return len(row) == empty_cells

    while row_index < len(rows):
        row = rows[row_index]
        line_with_value = row_index > 0 and row[group_by_column_index] == ''

        if line_with_value and not is_empty_row(row):
            rows[row_index - 1] = list(map(merge_cells,
                                           rows[row_index - 1],
                                           rows.pop(row_index)))
        else:
            row_index += 1

    return parsed_table


def listing(output_lines):
    """Return list of dicts with basic item info parsed from cli output."""

    items = []
    table_ = multi_line_row_table(output_lines)
    for row in table_['values']:
        item = {}
        for col_idx, col_key in enumerate(table_['headers']):
            item[col_key] = row[col_idx]
        items.append(item)
    return items


def details(output_lines):
    """Returns dict parsed from CLI output."""
    result = listing(output_lines)
    d = {}
    for item in result:
        d.update({item['Property']: item['Value']})
    return d


def is_microversion_supported(microversion):
    return (
        api_versions.APIVersion(CONF.min_api_microversion) <=
        api_versions.APIVersion(microversion) <=
        api_versions.APIVersion(CONF.max_api_microversion)
    )


def skip_if_microversion_not_supported(microversion):
    """Decorator for tests that are microversion-specific."""
    if not is_microversion_supported(microversion):
        reason = ("Skipped. Test requires microversion %s that is not "
                  "allowed to be used by configuration." % microversion)
        return testtools.skip(reason)
    return lambda f: f


def choose_matching_backend(share, pools, share_type):
    extra_specs = {}

    # convert extra-specs in provided type to dict format
    pair = [x.strip() for x in share_type['required_extra_specs'].split(':')]
    if len(pair) == 2:
        value = (True if str(pair[1]).lower() == 'true'
                 else False if str(pair[1]).lower() == 'false'
                 else pair[1])
        extra_specs[pair[0]] = value

    selected_pool = next(
        (x for x in pools if (x['Name'] != share['host'] and all(
            y in ast.literal_eval(x['Capabilities']).items() for y in
            extra_specs.items()))),
        None)

    return selected_pool['Name']


def share_network_subnets_are_supported():
    return is_microversion_supported('2.51')


def get_subnet_by_availability_zone_name(client, share_network_id, az_name):
    subnets = client.get_share_network_subnets(share_network_id)
    return next((subnet for subnet in subnets
                 if subnet['availability_zone'] == az_name), None)


def get_default_subnet(client, share_network_id):
    return get_subnet_by_availability_zone_name(client, share_network_id,
                                                'None')



# -*- coding: utf-8 -*-
# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ast
import ddt
from tempest.lib.common.utils import data_utils
from tempest.lib import exceptions as tempest_lib_exc
import time

from manilaclient import config
from manilaclient import exceptions
from manilaclient.tests.functional import base
from manilaclient.tests.functional import utils

SECURITY_SERVICE_UPDATE_VERSION = '2.63'
CONF = config.CONF


@ddt.ddt
class ShareNetworksReadWriteTest(base.BaseTestCase):

    def setUp(self):
        super(ShareNetworksReadWriteTest, self).setUp()
        self.name = data_utils.rand_name('autotest')
        self.description = 'fake_description'
        self.neutron_net_id = 'fake_neutron_net_id'
        self.neutron_subnet_id = 'fake_neutron_subnet_id'

        self.sn = self.create_share_network(
            name=self.name,
            description=self.description,
            neutron_net_id=self.neutron_net_id,
            neutron_subnet_id=self.neutron_subnet_id,
        )

    @ddt.data(
        {'name': data_utils.rand_name('autotest_share_network_name')},
        {'description': 'fake_description'},
        {'neutron_net_id': 'fake_neutron_net_id',
         'neutron_subnet_id': 'fake_neutron_subnet_id'},
    )
    def test_create_delete_share_network(self, net_data):
        share_subnet_support = utils.share_network_subnets_are_supported()
        share_subnet_fields = (
            ['neutron_net_id', 'neutron_subnet_id', 'availability_zone']
            if share_subnet_support else [])
        sn = self.create_share_network(cleanup_in_class=False, **net_data)
        default_subnet = (utils.get_default_subnet(self.user_client, sn['id'])
                          if share_subnet_support
                          else None)

        expected_data = {
            'name': 'None',
            'description': 'None',
            'neutron_net_id': 'None',
            'neutron_subnet_id': 'None',
        }
        expected_data.update(net_data)
        share_network_expected_data = [
            (k, v) for k, v in expected_data.items()
            if k not in share_subnet_fields]
        share_subnet_expected_data = [
            (k, v) for k, v in expected_data.items()
            if k in share_subnet_fields]

        for k, v in share_network_expected_data:
            self.assertEqual(v, sn[k])
        for k, v in share_subnet_expected_data:
            self.assertEqual(v, default_subnet[k])

        self.admin_client.delete_share_network(sn['id'])
        self.admin_client.wait_for_share_network_deletion(sn['id'])

    @utils.skip_if_microversion_not_supported('2.51')
    def test_create_delete_share_network_with_az(self):
        share_subnet_fields = (
            ['neutron_net_id', 'neutron_subnet_id', 'availability_zone'])
        az = self.user_client.list_availability_zones()[0]
        net_data = {
            'neutron_net_id': 'fake_neutron_net_id',
            'neutron_subnet_id': 'fake_neutron_subnet_id',
            'availability_zone': az['Name']
        }
        sn = self.create_share_network(cleanup_in_class=False, **net_data)
        default_subnet = utils.get_subnet_by_availability_zone_name(
            self.user_client, sn['id'], az['Name'])

        expected_data = {
            'name': 'None',
            'description': 'None',
            'neutron_net_id': 'None',
            'neutron_subnet_id': 'None',
            'availability_zone': 'None',
        }
        expected_data.update(net_data)
        share_network_expected_data = [
            (k, v) for k, v in expected_data.items()
            if k not in share_subnet_fields]
        share_subnet_expected_data = [
            (k, v) for k, v in expected_data.items()
            if k in share_subnet_fields]

        for k, v in share_network_expected_data:
            self.assertEqual(v, sn[k])
        for k, v in share_subnet_expected_data:
            self.assertEqual(v, default_subnet[k])

        self.admin_client.delete_share_network(sn['id'])
        self.admin_client.wait_for_share_network_deletion(sn['id'])

    def test_get_share_network_with_neutron_data(self):
        get = self.admin_client.get_share_network(self.sn['id'])

        self.assertEqual(self.name, get['name'])
        self.assertEqual(self.description, get['description'])
        if not utils.share_network_subnets_are_supported():
            self.assertEqual(self.neutron_net_id, get['neutron_net_id'])
            self.assertEqual(self.neutron_subnet_id, get['neutron_subnet_id'])

    def _get_expected_update_data(self, net_data, net_creation_data):
        # NOTE(dviroel): When subnets are supported, the outputs are converted
        # from string to literal structures in order to process the content of
        # 'share_network_subnets' field.
        default_return_value = (
            None if utils.share_network_subnets_are_supported() else 'None')

        expected_nn_id = (
            default_return_value
            if net_data.get('neutron_net_id')
            else net_creation_data.get('neutron_net_id', default_return_value))
        expected_nsn_id = (
            default_return_value
            if net_data.get('neutron_subnet_id')
            else net_creation_data.get('neutron_subnet_id',
                                       default_return_value))
        return expected_nn_id, expected_nsn_id

    @ddt.data(
        ({'name': data_utils.rand_name('autotest_share_network_name')}, {}),
        ({'description': 'fake_description'}, {}),
        ({'neutron_net_id': 'fake_neutron_net_id',
          'neutron_subnet_id': 'fake_neutron_subnet_id'}, {}),
        ({'name': '""'}, {}),
        ({'description': '""'}, {}),
        ({'neutron_net_id': '""'},
         {'neutron_net_id': 'fake_nn_id', 'neutron_subnet_id': 'fake_nsn_id'}),
        ({'neutron_subnet_id': '""'},
         {'neutron_net_id': 'fake_nn_id', 'neutron_subnet_id': 'fake_nsn_id'})
    )
    @ddt.unpack
    def test_create_update_share_network(self, net_data, net_creation_data):
        sn = self.create_share_network(
            cleanup_in_class=False, **net_creation_data)

        update = self.admin_client.update_share_network(sn['id'], **net_data)

        expected_nn_id, expected_nsn_id = self._get_expected_update_data(
            net_data, net_creation_data)

        expected_data = {
            'name': 'None',
            'description': 'None',
            'neutron_net_id': expected_nn_id,
            'neutron_subnet_id': expected_nsn_id,
        }
        subnet_keys = []
        if utils.share_network_subnets_are_supported():
            subnet_keys = ['neutron_net_id', 'neutron_subnet_id']
            subnet = ast.literal_eval(update['share_network_subnets'])

        update_values = dict([(k, v) for k, v in net_data.items()
                              if v != '""'])
        expected_data.update(update_values)

        for k, v in expected_data.items():
            if k in subnet_keys:
                self.assertEqual(v, subnet[0][k])
            else:
                self.assertEqual(v, update[k])

        self.admin_client.delete_share_network(sn['id'])
        self.admin_client.wait_for_share_network_deletion(sn['id'])

    @ddt.data(True, False)
    def test_list_share_networks(self, all_tenants):
        share_networks = self.admin_client.list_share_networks(all_tenants)

        self.assertTrue(
            any(self.sn['id'] == sn['id'] for sn in share_networks))
        for sn in share_networks:
            self.assertEqual(2, len(sn))
            self.assertIn('id', sn)
            self.assertIn('name', sn)

    def test_list_share_networks_select_column(self):
        share_networks = self.admin_client.list_share_networks(columns="id")
        self.assertTrue(any(s['Id'] is not None for s in share_networks))
        self.assertTrue(all('Name' not in s for s in share_networks))
        self.assertTrue(all('name' not in s for s in share_networks))

    def _list_share_networks_with_filters(self, filters):
        assert_subnet_fields = utils.share_network_subnets_are_supported()
        share_subnet_fields = (['neutron_subnet_id', 'neutron_net_id']
                               if assert_subnet_fields
                               else [])
        share_network_filters = [(k, v) for k, v in filters.items()
                                 if k not in share_subnet_fields]
        share_network_subnet_filters = [(k, v) for k, v in filters.items()
                                        if k in share_subnet_fields]
        share_networks = self.admin_client.list_share_networks(filters=filters)

        self.assertGreater(len(share_networks), 0)
        self.assertTrue(
            any(self.sn['id'] == sn['id'] for sn in share_networks))
        for sn in share_networks:
            try:
                share_network = self.admin_client.get_share_network(sn['id'])
                default_subnet = (
                    utils.get_default_subnet(self.user_client, sn['id'])
                    if assert_subnet_fields
                    else None)
            except tempest_lib_exc.NotFound:
                # NOTE(vponomaryov): Case when some share network was deleted
                # between our 'list' and 'get' requests. Skip such case.
                continue
            for k, v in share_network_filters:
                self.assertIn(k, share_network)
                self.assertEqual(v, share_network[k])
            for k, v in share_network_subnet_filters:
                self.assertIn(k, default_subnet)
                self.assertEqual(v, default_subnet[k])

    def test_list_share_networks_filter_by_project_id(self):
        project_id = self.admin_client.get_project_id(
            self.admin_client.tenant_name)
        filters = {'project_id': project_id}
        self._list_share_networks_with_filters(filters)

    def test_list_share_networks_filter_by_name(self):
        filters = {'name': self.name}
        self._list_share_networks_with_filters(filters)

    def test_list_share_networks_filter_by_description(self):
        filters = {'description': self.description}
        self._list_share_networks_with_filters(filters)

    def test_list_share_networks_filter_by_neutron_net_id(self):
        filters = {'neutron_net_id': self.neutron_net_id}
        self._list_share_networks_with_filters(filters)

    def test_list_share_networks_filter_by_neutron_subnet_id(self):
        filters = {'neutron_subnet_id': self.neutron_subnet_id}
        self._list_share_networks_with_filters(filters)

    @ddt.data('name', 'description')
    def test_list_share_networks_filter_by_inexact(self, option):
        self.create_share_network(
            name=data_utils.rand_name('autotest_inexact'),
            description='fake_description_inexact',
            neutron_net_id='fake_neutron_net_id',
            neutron_subnet_id='fake_neutron_subnet_id',
        )

        filters = {option + '~': 'inexact'}
        share_networks = self.admin_client.list_share_networks(
            filters=filters)

        self.assertGreater(len(share_networks), 0)

    def test_list_share_networks_by_inexact_unicode_option(self):
        self.create_share_network(
            name=u'网络名称',
            description=u'网络描述',
            neutron_net_id='fake_neutron_net_id',
            neutron_subnet_id='fake_neutron_subnet_id',
        )

        filters = {'name~': u'名称'}
        share_networks = self.admin_client.list_share_networks(
            filters=filters)

        self.assertGreater(len(share_networks), 0)

        filters = {'description~': u'描述'}
        share_networks = self.admin_client.list_share_networks(
            filters=filters)

        self.assertGreater(len(share_networks), 0)

    def test_share_network_reset_status(self):
        share_network = self.create_share_network(
            client=self.user_client,
            name='cool_net_name',
            description='fakedescription',
            neutron_net_id='fake_neutron_net_id',
            neutron_subnet_id='fake_neutron_subnet_id',
        )

        # Admin operation
        self.admin_client.share_network_reset_state(
            share_network['id'], 'error',
            microversion=SECURITY_SERVICE_UPDATE_VERSION)

        self.user_client.wait_for_resource_status(
            share_network['id'], 'error',
            microversion=SECURITY_SERVICE_UPDATE_VERSION,
            resource_type="share_network")

    def test_share_network_security_service_add(self):
        share_network = self.create_share_network(
            client=self.user_client,
            name='cool_net_name',
            description='fakedescription',
            neutron_net_id='fake_neutron_net_id',
            neutron_subnet_id='fake_neutron_subnet_id',
        )
        new_security_service = self.create_security_service(
            client=self.user_client)

        check_result = (
            self.user_client.share_network_security_service_add_check(
                share_network['id'],
                security_service_id=new_security_service['id']))

        self.assertEqual(check_result['compatible'], 'True')

        self.user_client.share_network_security_service_add(
            share_network['id'], new_security_service['id'])

        network_services = (
            self.user_client.share_network_security_service_list(
                share_network['id']))

        self.assertEqual(len(network_services), 1)
        self.assertEqual(
            network_services[0]['id'], new_security_service['id'])

    def test_share_network_security_service_update(self):
        share_network = self.create_share_network(
            client=self.user_client,
            name='cool_net_name',
            description='fakedescription',
            neutron_net_id='fake_neutron_net_id',
            neutron_subnet_id='fake_neutron_subnet_id',
        )
        current_name = 'current'
        new_name = 'new'
        current_security_service = self.create_security_service(
            client=self.user_client, name=current_name)
        new_security_service = self.create_security_service(
            client=self.user_client, name=new_name)

        check_result = (
            self.user_client.share_network_security_service_add_check(
                share_network['id'], current_security_service['id']))

        self.assertEqual(check_result['compatible'], 'True')

        self.user_client.share_network_security_service_add(
            share_network['id'], current_security_service['id'])

        network_services = (
            self.user_client.share_network_security_service_list(
                share_network['id']))

        self.assertEqual(len(network_services), 1)
        self.assertEqual(network_services[0]['name'], current_name)

        check_result = (
            self.user_client.share_network_security_service_update_check(
                share_network['id'], current_security_service['id'],
                new_security_service['id']))

        self.assertEqual(check_result['compatible'], 'True')

        self.user_client.share_network_security_service_update(
            share_network['id'], current_security_service['id'],
            new_security_service['id'])

        network_services = (
            self.user_client.share_network_security_service_list(
                share_network['id']))

        self.assertEqual(len(network_services), 1)
        self.assertEqual(network_services[0]['name'], new_name)

    def test_share_network_subnet_create_check(self):
        share_network = self.create_share_network(
            client=self.user_client,
            description='fakedescription',
        )

        check_result = (
            self.user_client.share_network_subnet_create_check(
                share_network['id'], neutron_net_id='fake_neutron_net_id',
                neutron_subnet_id='fake_neutron_subnet_id'))

        self.assertEqual(check_result['compatible'], 'True')

    @ddt.data(
        {'neutron_net_id': None, 'neutron_subnet_id': 'fake_subnet_id'},
        {'neutron_net_id': 'fake_net_id', 'neutron_subnet_id': None},
        {'availability_zone': 'invalid_availability_zone'},
    )
    def test_check_add_share_network_subnet_with_invalid_params(self, params):
        self.assertRaises(
            tempest_lib_exc.CommandFailed,
            self.user_client.share_network_subnet_create_check,
            self.sn['id'],
            **params)

    def test_check_add_share_network_subnet_to_invalid_share_network(self):
        self.assertRaises(
            tempest_lib_exc.CommandFailed,
            self.user_client.share_network_subnet_create_check,
            'invalid_share_network',
            self.neutron_net_id,
            self.neutron_subnet_id)


class ShareNetworkSecurityServiceCheckReadWriteTests(base.BaseTestCase):
    protocol = None

    def setUp(self):
        super(ShareNetworkSecurityServiceCheckReadWriteTests, self).setUp()
        if self.protocol not in CONF.enable_protocols:
            message = "%s tests are disabled." % self.protocol
            raise self.skipException(message)
        self.client = self.get_user_client()
        if not self.client.share_network:
            message = "Can run only with DHSS=True mode"
            raise self.skipException(message)

    def _wait_for_update_security_service_compatible_result(
            self, share_network, current_security_service,
            new_security_service=None):
        compatible_expected_result = 'True'
        check_is_compatible = 'None'
        tentatives = 0

        # There might be a delay from the time the check is requested until
        # the backend has performed all checks
        while check_is_compatible != compatible_expected_result:
            tentatives += 1
            if not new_security_service:
                check_is_compatible = (
                    self.user_client.share_network_security_service_add_check(
                        share_network['id'],
                        current_security_service['id']))['compatible']
            else:
                check_is_compatible = (
                    (self.user_client.
                        share_network_security_service_update_check(
                            share_network['id'],
                            current_security_service['id'],
                            new_security_service['id'])))['compatible']
            if tentatives > 3:
                timeout_message = (
                    "Share network security service add/update check did not "
                    "reach 'compatible=True' within 15 seconds.")
                raise exceptions.TimeoutException(message=timeout_message)
            time.sleep(5)

    def test_check_if_security_service_can_be_added_to_share_network_in_use(
            self):
        share_network = self.create_share_network(
            client=self.user_client,
            description='fakedescription',
            neutron_net_id='fake_neutron_net_id',
            neutron_subnet_id='fake_neutron_subnet_id',
        )
        # Create a share so we can be sure that a share server will exist and
        # the check will be performed in the backends
        self.create_share(
            self.protocol, client=self.user_client,
            share_network=share_network['id'])

        current_security_service = self.create_security_service(
            client=self.user_client)

        check_result = (
            self.user_client.share_network_security_service_add_check(
                share_network['id'],
                current_security_service['id']))

        self.assertEqual(check_result['compatible'], 'None')

        self._wait_for_update_security_service_compatible_result(
            share_network, current_security_service)

    def test_add_and_update_security_service_when_share_network_is_in_use(
            self):
        share_network = self.create_share_network(
            client=self.user_client,
            name='cool_net_name',
            description='fakedescription',
            neutron_net_id='fake_neutron_net_id',
            neutron_subnet_id='fake_neutron_subnet_id',
        )

        # Create a share so we can be sure that a share server will exist and
        # the check will be performed in the backends
        self.create_share(
            self.protocol, name='fake_share_name',
            share_network=share_network['id'], client=self.user_client)

        current_security_service = self.create_security_service(
            client=self.user_client, name='current_security_service')
        new_security_service = self.create_security_service(
            client=self.user_client, name='new_security_service')

        check_result = (
            self.user_client.share_network_security_service_add_check(
                share_network['id'], current_security_service['id']))

        self.assertEqual(check_result['compatible'], 'None')

        self._wait_for_update_security_service_compatible_result(
            share_network, current_security_service)

        self.user_client.share_network_security_service_add(
            share_network['id'], current_security_service['id'])

        network_services = (
            self.user_client.share_network_security_service_list(
                share_network['id']))

        self.assertEqual(len(network_services), 1)
        self.assertEqual(
            network_services[0]['name'], current_security_service['name'])

        self.user_client.wait_for_resource_status(
            share_network['id'], 'active',
            microversion=SECURITY_SERVICE_UPDATE_VERSION,
            resource_type="share_network")

        check_result = (
            self.user_client.share_network_security_service_update_check(
                share_network['id'], current_security_service['id'],
                new_security_service['id']))

        self.assertEqual(check_result['compatible'], 'None')

        self._wait_for_update_security_service_compatible_result(
            share_network, current_security_service,
            new_security_service=new_security_service)

        self.user_client.share_network_security_service_update(
            share_network['id'], current_security_service['id'],
            new_security_service['id'])

        network_services = (
            self.user_client.share_network_security_service_list(
                share_network['id']))

        self.assertEqual(len(network_services), 1)
        self.assertEqual(
            network_services[0]['name'], new_security_service['name'])

        self.user_client.wait_for_resource_status(
            share_network['id'], 'active',
            microversion=SECURITY_SERVICE_UPDATE_VERSION,
            resource_type="share_network")


base_security_service_check = ShareNetworkSecurityServiceCheckReadWriteTests


class ShareNetworkSecServiceCheckRWNFSTest(base_security_service_check):
    protocol = 'nfs'


class ShareNetworkSecServiceCheckRWTestsCIFSTest(base_security_service_check):
    protocol = 'cifs'



# Copyright 2015 Mirantis inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt

from manilaclient import api_versions
from manilaclient import extension
from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes
from manilaclient.v2 import share_instance_export_locations


extensions = [
    extension.Extension('share_instance_export_locations',
                        share_instance_export_locations),
]
cs = fakes.FakeClient(extensions=extensions)


@ddt.ddt
class ShareInstanceExportLocationsTest(utils.TestCase):

    def _get_manager(self, microversion):
        version = api_versions.APIVersion(microversion)
        mock_microversion = mock.Mock(api_version=version)
        return (
            share_instance_export_locations.ShareInstanceExportLocationManager(
                api=mock_microversion)
        )

    def test_list_of_export_locations(self):
        share_instance_id = '1234'
        cs.share_instance_export_locations.list(
            share_instance_id, search_opts=None)
        cs.assert_called(
            'GET', '/share_instances/%s/export_locations' % share_instance_id)

    def test_get_single_export_location(self):
        share_instance_id = '1234'
        el_uuid = 'fake_el_uuid'
        cs.share_instance_export_locations.get(share_instance_id, el_uuid)
        cs.assert_called(
            'GET',
            ('/share_instances/%(share_instance_id)s/export_locations/'
             '%(el_uuid)s') % {
                 'share_instance_id': share_instance_id, 'el_uuid': el_uuid})



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from unittest import mock

from osc_lib import exceptions
from osc_lib import utils as oscutils

from manilaclient import api_versions
from manilaclient.common import cliutils

from manilaclient.osc import utils
from manilaclient.osc.v2 import share_replicas as osc_share_replicas

from manilaclient.tests.unit.osc import osc_utils
from manilaclient.tests.unit.osc.v2 import fakes as manila_fakes


class TestShareReplica(manila_fakes.TestShare):

    def setUp(self):
        super(TestShareReplica, self).setUp()

        self.shares_mock = self.app.client_manager.share.shares
        self.shares_mock.reset_mock()

        self.replicas_mock = self.app.client_manager.share.share_replicas
        self.replicas_mock.reset_mock()
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            api_versions.MAX_VERSION)

        self.replica_el_mock = (
            self.app.client_manager
                .share.share_replica_export_locations)
        self.replica_el_mock.reset_mock()


class TestShareReplicaCreate(TestShareReplica):

    def setUp(self):
        super(TestShareReplicaCreate, self).setUp()

        self.share = manila_fakes.FakeShare.create_one_share()
        self.shares_mock.get.return_value = self.share

        self.share_replica = (
            manila_fakes.FakeShareReplica.create_one_replica(
                attrs={
                    'availability_zone': 'manila-zone-1',
                    'status': 'available'}
            ))
        self.replicas_mock.create.return_value = self.share_replica
        self.replicas_mock.get.return_value = self.share_replica

        self.cmd = osc_share_replicas.CreateShareReplica(self.app, None)

        self.data = tuple(self.share_replica._info.values())
        self.columns = tuple(self.share_replica._info.keys())

    def test_share_replica_create_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(
            osc_utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_share_replica_create(self):
        arglist = [
            self.share.id
        ]
        verifylist = [
            ('share', self.share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.replicas_mock.create.assert_called_with(
            share=self.share,
            availability_zone=None
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_share_replica_create_az(self):
        arglist = [
            self.share.id,
            '--availability-zone', self.share.availability_zone
        ]
        verifylist = [
            ('share', self.share.id),
            ('availability_zone', self.share.availability_zone)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.replicas_mock.create.assert_called_with(
            share=self.share,
            availability_zone=self.share.availability_zone
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_share_replica_create_scheduler_hint_valid(self):
        arglist = [
            self.share.id,
            '--availability-zone', self.share.availability_zone,
            '--scheduler-hint', ('only_host=host1@backend1#pool1'),
        ]
        verifylist = [
            ('share', self.share.id),
            ('availability_zone', self.share.availability_zone),
            ('scheduler_hint', {'only_host': 'host1@backend1#pool1'})
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.replicas_mock.create.assert_called_with(
            share=self.share,
            availability_zone=self.share.availability_zone,
            scheduler_hints={'only_host': 'host1@backend1#pool1'}
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_share_replica_create_scheduler_hint_invalid_hint(self):
        arglist = [
            self.share.id,
            '--availability-zone', self.share.availability_zone,
            '--scheduler-hint', 'fake_hint=host1@backend1#pool1'
        ]
        verifylist = [
            ('share', self.share.id),
            ('availability_zone', self.share.availability_zone),
            ('scheduler_hint', {'fake_hint': 'host1@backend1#pool1'})
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)

    def test_share_replica_create_scheduler_hint_invalid_version(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.66")

        arglist = [
            self.share.id,
            '--availability-zone', self.share.availability_zone,
            '--scheduler-hint', 'only_host=host1@backend1#pool1'
        ]
        verifylist = [
            ('share', self.share.id),
            ('availability_zone', self.share.availability_zone),
            ('scheduler_hint', {'only_host': 'host1@backend1#pool1'})
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)

    def test_share_replica_create_share_network(self):
        self.app.client_manager.share.api_version = api_versions.APIVersion(
            "2.72")

        arglist = [
            self.share.id,
            '--availability-zone', self.share.availability_zone,
            '--share-network', self.share.share_network_id
        ]
        verifylist = [
            ('share', self.share.id),
            ('availability_zone', self.share.availability_zone),
            ('share_network', self.share.share_network_id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)
        if self.share.share_network_id:
            self.replicas_mock.create.assert_called_with(
                share=self.share,
                availability_zone=self.share.availability_zone,
                share_network=self.share.share_network_id
            )
        else:
            self.replicas_mock.create.assert_called_with(
                share=self.share,
                availability_zone=self.share.availability_zone,
            )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    def test_share_replica_create_wait(self):
        arglist = [
            self.share.id,
            '--wait'
        ]
        verifylist = [
            ('share', self.share.id),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.replicas_mock.create.assert_called_with(
            share=self.share,
            availability_zone=None
        )

        self.replicas_mock.get.assert_called_with(self.share_replica.id)
        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)

    @mock.patch('manilaclient.osc.v2.share_replicas.LOG')
    def test_share_replica_create_wait_exception(self, mock_logger):
        arglist = [
            self.share.id,
            '--wait'
        ]
        verifylist = [
            ('share', self.share.id),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_status', return_value=False):
            columns, data = self.cmd.take_action(parsed_args)

            self.replicas_mock.create.assert_called_with(
                share=self.share,
                availability_zone=None
            )

            mock_logger.error.assert_called_with(
                "ERROR: Share replica is in error state.")

            self.replicas_mock.get.assert_called_with(self.share_replica.id)
            self.assertCountEqual(self.columns, columns)
            self.assertCountEqual(self.data, data)


class TestShareReplicaDelete(TestShareReplica):

    def setUp(self):
        super(TestShareReplicaDelete, self).setUp()

        self.share_replica = (
            manila_fakes.FakeShareReplica.create_one_replica())
        self.replicas_mock.get.return_value = self.share_replica

        self.cmd = osc_share_replicas.DeleteShareReplica(self.app, None)

    def test_share_replica_delete_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(osc_utils.ParserException,
                          self.check_parser, self.cmd, arglist, verifylist)

    def test_share_replica_delete(self):
        arglist = [
            self.share_replica.id
        ]
        verifylist = [
            ('replica', [self.share_replica.id])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.replicas_mock.delete.assert_called_with(
            self.share_replica,
            force=False)
        self.assertIsNone(result)

    def test_share_replica_delete_force(self):
        arglist = [
            self.share_replica.id,
            '--force'
        ]
        verifylist = [
            ('replica', [self.share_replica.id]),
            ('force', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.replicas_mock.delete.assert_called_with(
            self.share_replica,
            force=True)
        self.assertIsNone(result)

    def test_share_replica_delete_multiple(self):
        share_replicas = (
            manila_fakes.FakeShareReplica.create_share_replicas(
                count=2))
        arglist = [
            share_replicas[0].id,
            share_replicas[1].id
        ]
        verifylist = [
            ('replica', [share_replicas[0].id, share_replicas[1].id])
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.assertEqual(self.replicas_mock.delete.call_count,
                         len(share_replicas))
        self.assertIsNone(result)

    def test_share_snapshot_delete_exception(self):
        arglist = [
            self.share_replica.id
        ]
        verifylist = [
            ('replica', [self.share_replica.id])
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.replicas_mock.delete.side_effect = exceptions.CommandError()
        self.assertRaises(exceptions.CommandError,
                          self.cmd.take_action,
                          parsed_args)

    def test_share_replica_delete_wait(self):
        arglist = [
            self.share_replica.id,
            '--wait'
        ]
        verifylist = [
            ('replica', [self.share_replica.id]),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_delete', return_value=True):
            result = self.cmd.take_action(parsed_args)

            self.replicas_mock.delete.assert_called_with(
                self.share_replica,
                force=False)
            self.replicas_mock.get.assert_called_with(self.share_replica.id)
            self.assertIsNone(result)

    def test_share_replica_delete_wait_exception(self):
        arglist = [
            self.share_replica.id,
            '--wait'
        ]
        verifylist = [
            ('replica', [self.share_replica.id]),
            ('wait', True)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        with mock.patch('osc_lib.utils.wait_for_delete', return_value=False):
            self.assertRaises(
                exceptions.CommandError,
                self.cmd.take_action,
                parsed_args
            )


class TestShareReplicaList(TestShareReplica):

    columns = [
        'id',
        'status',
        'replica_state',
        'share_id',
        'host',
        'availability_zone',
        'updated_at',
    ]

    column_headers = utils.format_column_headers(columns)

    def setUp(self):
        super(TestShareReplicaList, self).setUp()

        self.share = manila_fakes.FakeShare.create_one_share()
        self.shares_mock.get.return_value = self.share

        self.replicas_list = (
            manila_fakes.FakeShareReplica.create_share_replicas(
                count=2))
        self.replicas_mock.list.return_value = self.replicas_list

        self.values = (oscutils.get_dict_properties(
            i._info, self.columns) for i in self.replicas_list)

        self.cmd = osc_share_replicas.ListShareReplica(self.app, None)

    def test_share_replica_list(self):
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.replicas_mock.list.assert_called_with(share=None)

        self.assertEqual(self.column_headers, columns)
        self.assertEqual(list(self.values), list(data))

    def test_share_replica_list_for_share(self):
        arglist = [
            '--share', self.share.id
        ]
        verifylist = [
            ('share', self.share.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.replicas_mock.list.assert_called_with(share=self.share)

        self.assertEqual(self.column_headers, columns)
        self.assertEqual(list(self.values), list(data))


class TestShareReplicaShow(TestShareReplica):

    def setUp(self):
        super(TestShareReplicaShow, self).setUp()

        self.share_replica = (
            manila_fakes.FakeShareReplica.create_one_replica()
        )
        self.replicas_mock.get.return_value = self.share_replica

        self.replica_el_list = (
            manila_fakes.FakeShareExportLocation.
            create_share_export_locations(count=2)
        )

        self.replica_el_mock.list.return_value = (
            self.replica_el_list)

        self.cmd = osc_share_replicas.ShowShareReplica(self.app, None)

        self.share_replica._info['export_locations'] = (
            cliutils.convert_dict_list_to_string(
                self.replica_el_list))

        self.data = tuple(self.share_replica._info.values())
        self.columns = tuple(self.share_replica._info.keys())

    def test_share_replica_show_missing_args(self):
        arglist = []
        verifylist = []

        self.assertRaises(
            osc_utils.ParserException,
            self.check_parser, self.cmd, arglist, verifylist)

    def test_share_replica_show(self):
        arglist = [
            self.share_replica.id
        ]
        verifylist = [
            ('replica', self.share_replica.id)
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        columns, data = self.cmd.take_action(parsed_args)

        self.replicas_mock.get.assert_called_with(
            self.share_replica.id
        )

        self.assertCountEqual(self.columns, columns)
        self.assertCountEqual(self.data, data)


class TestShareReplicaSet(TestShareReplica):

    def setUp(self):
        super(TestShareReplicaSet, self).setUp()

        self.share_replica = (
            manila_fakes.FakeShareReplica.create_one_replica()
        )
        self.replicas_mock.get.return_value = self.share_replica

        self.cmd = osc_share_replicas.SetShareReplica(self.app, None)

    def test_share_replica_set_replica_state(self):
        new_replica_state = 'in_sync'
        arglist = [
            self.share_replica.id,
            '--replica-state', new_replica_state
        ]
        verifylist = [
            ('replica', self.share_replica.id),
            ('replica_state', new_replica_state)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.replicas_mock.reset_replica_state.assert_called_with(
            self.share_replica,
            new_replica_state)
        self.assertIsNone(result)

    def test_share_replica_set_replica_state_exception(self):
        new_replica_state = 'in_sync'
        arglist = [
            self.share_replica.id,
            '--replica-state', new_replica_state
        ]
        verifylist = [
            ('replica', self.share_replica.id),
            ('replica_state', new_replica_state)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.replicas_mock.reset_replica_state.side_effect = Exception()

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)

    def test_share_replica_set_status(self):
        new_status = 'available'
        arglist = [
            self.share_replica.id,
            '--status', new_status
        ]
        verifylist = [
            ('replica', self.share_replica.id),
            ('status', new_status)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.replicas_mock.reset_state.assert_called_with(
            self.share_replica,
            new_status)
        self.assertIsNone(result)

    def test_share_replica_set_status_exception(self):
        new_status = 'available'
        arglist = [
            self.share_replica.id,
            '--status', new_status
        ]
        verifylist = [
            ('replica', self.share_replica.id),
            ('status', new_status)
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.replicas_mock.reset_state.side_effect = Exception()

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)

    def test_share_replica_set_nothing_defined(self):
        arglist = [
            self.share_replica.id,
        ]
        verifylist = [
            ('replica', self.share_replica.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)


class TestShareReplicaPromote(TestShareReplica):

    def setUp(self):
        super(TestShareReplicaPromote, self).setUp()

        self.share_replica = (
            manila_fakes.FakeShareReplica.create_one_replica()
        )
        self.replicas_mock.get.return_value = self.share_replica

        self.cmd = osc_share_replicas.PromoteShareReplica(
            self.app, None)

    def test_share_replica_promote(self):
        arglist = [
            self.share_replica.id,
        ]
        verifylist = [
            ('replica', self.share_replica.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.replicas_mock.promote.assert_called_with(
            self.share_replica)
        self.assertIsNone(result)

    def test_share_replica_promote_exception(self):
        arglist = [
            self.share_replica.id,
        ]
        verifylist = [
            ('replica', self.share_replica.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.replicas_mock.promote.side_effect = Exception()

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)


class TestShareReplicaResync(TestShareReplica):

    def setUp(self):
        super(TestShareReplicaResync, self).setUp()

        self.share_replica = (
            manila_fakes.FakeShareReplica.create_one_replica()
        )
        self.replicas_mock.get.return_value = self.share_replica

        self.cmd = osc_share_replicas.ResyncShareReplica(
            self.app, None)

    def test_share_replica_resync(self):
        arglist = [
            self.share_replica.id,
        ]
        verifylist = [
            ('replica', self.share_replica.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.replicas_mock.resync.assert_called_with(
            self.share_replica)
        self.assertIsNone(result)

    def test_share_replica_resync_exception(self):
        arglist = [
            self.share_replica.id,
        ]
        verifylist = [
            ('replica', self.share_replica.id),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.replicas_mock.resync.side_effect = Exception()

        self.assertRaises(
            exceptions.CommandError,
            self.cmd.take_action,
            parsed_args)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from manilaclient import config
from manilaclient.tests.functional.osc import base
from tempest.lib.common.utils import data_utils

CONF = config.CONF


class ShareReplicasCLITest(base.OSCClientTestBase):

    def _create_share_and_replica(self, add_cleanup=True):
        replication_type = CONF.replication_type
        share_type = self.create_share_type(
            data_utils.rand_name('test_share_type'),
            dhss=True,
            extra_specs={'replication_type': replication_type})
        share_network = self.create_share_network(name='test_share_network')
        share = self.create_share(share_type=share_type['name'],
                                  share_network=share_network['id'])
        replica = self.create_share_replica(share['id'],
                                            share_network=share_network['id'],
                                            wait=True,
                                            add_cleanup=add_cleanup)
        return replica

    def test_share_replica_create(self):
        share_replica = self._create_share_and_replica()
        share_replica_list = self.listing_result('share replica', 'list')
        self.assertTableStruct(share_replica_list, [
            'ID',
            'Status',
            'Share ID',
        ])
        self.assertIn(share_replica['id'],
                      [item['ID'] for item in share_replica_list])

    def test_share_replica_delete(self):
        share_replica = self._create_share_and_replica(add_cleanup=False)
        self.openstack(
            f'share replica delete {share_replica["id"]}'
        )
        self.check_object_deleted('share replica', share_replica["id"])



# Copyright 2016 Clinton Knight
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

import ddt

import manilaclient
from manilaclient import exceptions
from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes as fake
from manilaclient.v2 import share_group_types as types


@ddt.ddt
class ShareGroupTypeTest(utils.TestCase):

    def setUp(self):
        super(ShareGroupTypeTest, self).setUp()
        self.manager = types.ShareGroupTypeManager(fake.FakeClient())
        self.fake_group_specs = {'key1': 'value1', 'key2': 'value2'}
        self.fake_share_group_type_info = {
            'id': fake.ShareGroupType.id,
            'share_types': [fake.ShareType.id],
            'name': fake.ShareGroupType.name,
            'is_public': fake.ShareGroupType.is_public,
            'group_specs': self.fake_group_specs,
        }
        self.share_group_type = types.ShareGroupType(
            self.manager, self.fake_share_group_type_info, loaded=True)

    def test_repr(self):
        result = str(self.share_group_type)

        self.assertEqual(
            '<Share Group Type: %s>' % fake.ShareGroupType.name, result)

    @ddt.data((True, True), (False, False), (None, 'N/A'))
    @ddt.unpack
    def test_is_public(self, is_public, expected):
        fake_share_group_type_info = {'name': 'fake_name'}
        if is_public is not None:
            fake_share_group_type_info['is_public'] = is_public
        share_group_type = types.ShareGroupType(
            self.manager, fake_share_group_type_info, loaded=True)

        result = share_group_type.is_public

        self.assertEqual(expected, result)

    def test_get_keys(self):
        self.mock_object(self.manager.api.client, 'get')

        result = self.share_group_type.get_keys()

        self.assertEqual(self.fake_group_specs, result)
        self.assertFalse(self.manager.api.client.get.called)

    def test_get_keys_force_api_call(self):
        share_group_type = types.ShareGroupType(
            self.manager, self.fake_share_group_type_info, loaded=True)
        share_group_type._group_specs = {}
        self.manager.api.client.get = mock.Mock(return_value=(
            None, self.fake_share_group_type_info))

        result = share_group_type.get_keys(prefer_resource_data=False)

        self.assertEqual(self.fake_group_specs, result)
        self.manager.api.client.get.assert_called_once_with(
            types.GROUP_SPECS_RESOURCES_PATH % fake.ShareGroupType.id)

    def test_set_keys(self):
        mock_manager_create = self.mock_object(
            self.manager, '_create',
            mock.Mock(return_value=self.fake_group_specs))

        result = self.share_group_type.set_keys(self.fake_group_specs)

        self.assertEqual(result, self.fake_group_specs)
        expected_body = {'group_specs': self.fake_group_specs}
        mock_manager_create.assert_called_once_with(
            types.GROUP_SPECS_RESOURCES_PATH % fake.ShareGroupType.id,
            expected_body, types.GROUP_SPECS_RESOURCES_NAME, return_raw=True)

    def test_unset_keys(self):
        mock_manager_delete = self.mock_object(
            self.manager, '_delete', mock.Mock(return_value=None))

        result = self.share_group_type.unset_keys(self.fake_group_specs.keys())

        self.assertIsNone(result)
        mock_manager_delete.assert_has_calls([
            mock.call(types.GROUP_SPECS_RESOURCE_PATH %
                      (fake.ShareGroupType.id, 'key1')),
            mock.call(types.GROUP_SPECS_RESOURCE_PATH %
                      (fake.ShareGroupType.id, 'key2')),
        ], any_order=True)

    def test_unset_keys_error(self):
        mock_manager_delete = self.mock_object(
            self.manager, '_delete', mock.Mock(return_value='error'))

        result = self.share_group_type.unset_keys(
            sorted(self.fake_group_specs.keys()))

        self.assertEqual('error', result)
        mock_manager_delete.assert_called_once_with(
            types.GROUP_SPECS_RESOURCE_PATH % (fake.ShareGroupType.id, 'key1'))


@ddt.ddt
class ShareGroupTypeManagerTest(utils.TestCase):

    def setUp(self):
        super(ShareGroupTypeManagerTest, self).setUp()
        self.manager = types.ShareGroupTypeManager(fake.FakeClient())
        self.fake_group_specs = {'key1': 'value1', 'key2': 'value2'}

    def test_create(self):
        fake_share_group_type = fake.ShareGroupType()
        mock_create = self.mock_object(
            self.manager, '_create',
            mock.Mock(return_value=fake_share_group_type))
        create_args = {
            'name': fake.ShareGroupType.name,
            'share_types': [fake.ShareType()],
            'is_public': False,
            'group_specs': self.fake_group_specs,
        }

        result = self.manager.create(**create_args)

        self.assertIs(fake_share_group_type, result)
        expected_body = {
            types.RESOURCE_NAME: {
                'name': fake.ShareGroupType.name,
                'share_types': [fake.ShareType().id],
                'is_public': False,
                'group_specs': self.fake_group_specs,
            },
        }
        mock_create.assert_called_once_with(
            types.RESOURCES_PATH, expected_body, types.RESOURCE_NAME)

    def test_create_no_share_type(self):
        create_args = {
            'name': fake.ShareGroupType.name,
            'share_types': [],
            'is_public': False,
            'group_specs': self.fake_group_specs,
        }

        self.assertRaises(ValueError, self.manager.create, **create_args)

    def test_create_using_unsupported_microversion(self):
        self.manager.api.api_version = manilaclient.API_MIN_VERSION

        self.assertRaises(exceptions.UnsupportedVersion, self.manager.create)

    def test_get(self):
        fake_share_group_type = fake.ShareGroupType()
        mock_get = self.mock_object(
            self.manager, '_get',
            mock.Mock(return_value=fake_share_group_type))

        result = self.manager.get(fake.ShareGroupType.id)

        self.assertIs(fake_share_group_type, result)
        mock_get.assert_called_once_with(
            types.RESOURCE_PATH % fake.ShareGroupType.id, types.RESOURCE_NAME)

    def test_list(self):
        fake_share_group_type = fake.ShareGroupType()
        mock_list = self.mock_object(
            self.manager, '_list',
            mock.Mock(return_value=[fake_share_group_type]))

        result = self.manager.list(search_opts=None)

        self.assertEqual([fake_share_group_type], result)
        mock_list.assert_called_once_with(
            types.RESOURCES_PATH + '?is_public=all', types.RESOURCES_NAME)

    def test_list_no_public(self):
        fake_share_group_type = fake.ShareGroupType()
        mock_list = self.mock_object(
            self.manager, '_list',
            mock.Mock(return_value=[fake_share_group_type]))

        result = self.manager.list(show_all=False)

        self.assertEqual([fake_share_group_type], result)
        mock_list.assert_called_once_with(
            types.RESOURCES_PATH, types.RESOURCES_NAME)

    def test_delete(self):
        mock_delete = self.mock_object(self.manager, '_delete')

        self.manager.delete(fake.ShareGroupType())

        mock_delete.assert_called_once_with(
            types.RESOURCE_PATH % fake.ShareGroupType.id)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import json

from manilaclient.tests.functional.osc import base


class ShareTypesCLITest(base.OSCClientTestBase):

    def test_share_type_create(self):
        name = 'test_share_type'
        description = 'Description'
        share_type = self.create_share_type(
            name=name, description=description)

        self.assertEqual(name, share_type["name"])
        self.assertEqual(description, share_type["description"])
        self.assertEqual('public', share_type["visibility"])

        share_types_list = self.listing_result('share type', 'list')
        self.assertIn(
            share_type["id"], [item['ID'] for item in share_types_list])

    def test_share_type_create_specs(self):
        share_type = self.create_share_type(
            snapshot_support=True,
            create_share_from_snapshot_support=True,
            revert_to_snapshot_support=True,
            mount_snapshot_support=True,
            extra_specs={
                "foo": "bar",
                "manila": "zorilla"
            },
            formatter='json')

        required_specs = share_type["required_extra_specs"]
        optional_specs = share_type["optional_extra_specs"]

        self.assertEqual(
            False, required_specs["driver_handles_share_servers"])
        self.assertEqual('True', optional_specs["snapshot_support"])
        self.assertEqual(
            'True', optional_specs["create_share_from_snapshot_support"])
        self.assertEqual(
            'True', optional_specs["revert_to_snapshot_support"])
        self.assertEqual('True', optional_specs["mount_snapshot_support"])
        self.assertEqual("bar", optional_specs["foo"])
        self.assertEqual("zorilla", optional_specs["manila"])

    def test_share_type_delete(self):
        share_type_1 = self.create_share_type(add_cleanup=False)
        share_type_2 = self.create_share_type(add_cleanup=False)

        self.openstack(
            f'share type delete {share_type_1["id"]} {share_type_2["id"]}'
        )

        self.check_object_deleted('share type', share_type_1["id"])
        self.check_object_deleted('share type', share_type_2["id"])

    def test_share_type_set(self):
        share_type = self.create_share_type()

        self.openstack(
            f'share type set {share_type["id"]} --description Description'
            ' --name Name --public false --extra-specs foo=bar'
        )

        share_type = json.loads(self.openstack(
            f'share type show {share_type["id"]} -f json'
        ))

        self.assertEqual('Description', share_type["description"])
        self.assertEqual('Name', share_type["name"])
        self.assertEqual('private', share_type["visibility"])
        self.assertEqual(
            'bar', share_type["optional_extra_specs"]["foo"])

    def test_share_type_unset(self):
        share_type = self.create_share_type(
            snapshot_support=True,
            extra_specs={'foo': 'bar'})

        self.openstack(
            f'share type unset {share_type["id"]} '
            'snapshot_support foo')

        share_type = json.loads(self.openstack(
            f'share type show {share_type["id"]} -f json'
        ))

        self.assertNotIn('foo', share_type["optional_extra_specs"])
        self.assertNotIn(
            'snapshot_support', share_type["optional_extra_specs"])

    def test_share_type_list(self):
        share_type_1 = self.create_share_type(public=False)
        share_type_2 = self.create_share_type(
            extra_specs={'foo': 'bar'})

        types_list = self.listing_result(
            'share type', 'list --all', client=self.admin_client)

        self.assertTableStruct(types_list, [
            'ID',
            'Name',
            'Visibility',
            'Is Default',
            'Required Extra Specs',
            'Optional Extra Specs',
            'Description'
        ])
        id_list = [item['ID'] for item in types_list]
        self.assertIn(share_type_1['id'], id_list)
        self.assertIn(share_type_2['id'], id_list)

        types_list = self.listing_result(
            'share type', 'list --extra-specs foo=bar')

        id_list = [item['ID'] for item in types_list]
        self.assertNotIn(share_type_1['id'], id_list)
        self.assertIn(share_type_2['id'], id_list)

    def test_share_type_show(self):
        share_type = self.create_share_type(
            extra_specs={'foo': 'bar'})

        result = json.loads(self.openstack(
            f'share type show {share_type["id"]} -f json'))

        self.assertEqual(share_type["name"], result["name"])
        self.assertEqual(
            share_type["visibility"], result["visibility"])
        self.assertEqual(
            share_type["is_default"], str(result["is_default"]))
        self.assertEqual(
            'bar', result["optional_extra_specs"]["foo"])



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import json

from manilaclient.tests.functional.osc import base


class ShareSnapshotCLITest(base.OSCClientTestBase):
    """Functional tests for share snpshot."""

    def test_share_snapshot_create(self):
        share = self.create_share()

        snapshot = self.create_snapshot(
            share=share['id'],
            name='Snap',
            description='Description')

        self.assertEqual(share["id"], snapshot["share_id"])
        self.assertEqual('Snap', snapshot["name"])
        self.assertEqual('Description', snapshot["description"])
        self.assertEqual('available', snapshot["status"])

        snapshots_list = self.listing_result('share snapshot', 'list')
        self.assertIn(snapshot['id'],
                      [item['ID'] for item in snapshots_list])

    def test_share_snapshot_delete(self):
        share = self.create_share()

        snapshot_1 = self.create_snapshot(
            share=share['id'], add_cleanup=False)
        snapshot_2 = self.create_snapshot(
            share=share['id'], add_cleanup=False)

        self.openstack(
            f'share snapshot delete {snapshot_1["id"]} {snapshot_2["id"]} '
            '--wait')

        self.check_object_deleted('share snapshot', snapshot_1["id"])
        self.check_object_deleted('share snapshot', snapshot_2["id"])

        snapshot_3 = self.create_snapshot(
            share=share['id'], add_cleanup=False)

        self.openstack(
            f'share snapshot set {snapshot_3["id"]} '
            '--status creating')

        self.openstack(
            f'share snapshot delete {snapshot_3["id"]} --wait --force')

        self.check_object_deleted('share snapshot', snapshot_3["id"])

    def test_share_snapshot_show(self):
        share = self.create_share()

        snapshot = self.create_snapshot(
            share=share['id'],
            name='Snap',
            description='Description')

        show_result = self.dict_result(
            'share snapshot', f'show {snapshot["id"]}')

        self.assertEqual(snapshot["id"], show_result["id"])
        self.assertEqual('Snap', show_result["name"])
        self.assertEqual('Description', show_result["description"])

    def test_share_snapshot_set(self):
        share = self.create_share()

        snapshot = self.create_snapshot(share=share['id'])

        self.openstack(
            f'share snapshot set {snapshot["id"]} '
            f'--name Snap --description Description')

        show_result = self.dict_result(
            'share snapshot ', f'show {snapshot["id"]}')

        self.assertEqual(snapshot['id'], show_result["id"])
        self.assertEqual('Snap', show_result["name"])
        self.assertEqual('Description', show_result["description"])

    def test_share_snapshot_unset(self):
        share = self.create_share()

        snapshot = self.create_snapshot(
            share=share['id'],
            name='Snap',
            description='Description')

        self.openstack(
            f'share snapshot unset {snapshot["id"]} --name --description')

        show_result = json.loads(self.openstack(
            f'share snapshot show -f json {snapshot["id"]}'))

        self.assertEqual(snapshot['id'], show_result["id"])
        self.assertIsNone(show_result["name"])
        self.assertIsNone(show_result["description"])

    def test_share_snapshot_list(self):
        share = self.create_share()

        snapshot_1 = self.create_snapshot(share=share['id'])
        snapshot_2 = self.create_snapshot(
            share=share['id'],
            description='Description')

        snapshots_list = self.listing_result(
            'share snapshot', f'list --name {snapshot_2["name"]} '
            f'--description {snapshot_2["description"]} --all-projects'
        )

        self.assertTableStruct(snapshots_list, [
            'ID',
            'Name',
            'Project ID'
        ])

        self.assertIn(snapshot_2["name"],
                      [snap["Name"] for snap in snapshots_list])
        self.assertEqual(1, len(snapshots_list))

        snapshots_list = self.listing_result(
            'share snapshot', f'list --share {share["name"]}')

        self.assertTableStruct(snapshots_list, [
            'ID',
            'Name'
        ])

        id_list = [snap["ID"] for snap in snapshots_list]
        self.assertIn(snapshot_1["id"], id_list)
        self.assertIn(snapshot_2["id"], id_list)

        snapshots_list = self.listing_result(
            'share snapshot',
            f'list --name~ {snapshot_2["name"][-3:]} '
            '--description~ Des --detail')

        self.assertTableStruct(snapshots_list, [
            'ID',
            'Name',
            'Status',
            'Description',
            'Created At',
            'Size',
            'Share ID',
            'Share Proto',
            'Share Size',
            'User ID'
        ])

        self.assertIn(snapshot_2["name"],
                      [snap["Name"] for snap in snapshots_list])
        self.assertEqual(
            snapshot_2["description"], snapshots_list[0]['Description'])
        self.assertEqual(1, len(snapshots_list))

    def test_share_snapshot_export_location_list(self):
        share = self.create_share()

        snapshot = self.create_snapshot(share=share['id'])

        export_location_list = self.listing_result(
            'share snapshot export location', f' list {snapshot["id"]}')

        self.assertTableStruct(export_location_list, [
            'ID',
            'Path'
        ])

    def test_share_snapshot_export_location_show(self):
        share = self.create_share()

        snapshot = self.create_snapshot(share=share['id'])

        export_location_list = self.listing_result(
            'share snapshot export location', f'list {snapshot["id"]}')

        export_location = self.dict_result(
            'share snapshot export location',
            f'show {snapshot["id"]} {export_location_list[0]["ID"]}')

        self.assertIn('id', export_location)
        self.assertIn('created_at', export_location)
        self.assertIn('is_admin_only', export_location)
        self.assertIn('path', export_location)
        self.assertIn('share_snapshot_instance_id', export_location)
        self.assertIn('updated_at', export_location)

    def test_share_snapshot_abandon_adopt(self):
        share = self.create_share()

        snapshot = self.create_snapshot(
            share=share['id'], add_cleanup=False)

        self.openstack(
            f'share snapshot abandon {snapshot["id"]} --wait')

        snapshots_list = self.listing_result('share snapshot', 'list')
        self.assertNotIn(snapshot['id'],
                         [item['ID'] for item in snapshots_list])

        snapshot = self.dict_result(
            'share snapshot',
            f'adopt {share["id"]} 10.0.0.1:/foo/path '
            f'--name Snap --description Zorilla --wait')

        snapshots_list = self.listing_result('share snapshot', 'list')
        self.assertIn(snapshot['id'],
                      [item['ID'] for item in snapshots_list])

        self.addCleanup(
            self.openstack,
            f'share snapshot delete {snapshot["id"]} --force --wait')



# Copyright 2018 Huawei Corporation.
# All Rights Reserved.
#
# Copyright 2018
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Interface for share access rules extension."""

from manilaclient import api_versions
from manilaclient import base

RESOURCE_PATH = '/share-access-rules/%s'
RESOURCE_NAME = 'access'

RESOURCES_METADATA_PATH = '/share-access-rules/%s/metadata'
RESOURCE_METADATA_PATH = '/share-access-rules/%s/metadata/%s'
RESOURCE_LIST_PATH = '/share-access-rules'


class ShareAccessRule(base.Resource):
    """A Share Access Rule."""

    def __repr__(self):
        return "<Share Access Rule: %s>" % self.id

    def delete(self):
        """"Delete this share access rule."""
        self.manager.delete(self)


class ShareAccessRuleManager(base.ManagerWithFind):
    """Manage :class:`ShareAccessRule` resources."""

    resource_class = ShareAccessRule

    @api_versions.wraps("2.45")
    def get(self, share_access_rule):
        """Get a specific share access rule.

        :param share_access_rule: either instance of ShareAccessRule, or text
           with UUID
        :rtype: :class:`ShareAccessRule`
        """
        share_access_rule_id = base.getid(share_access_rule)
        url = RESOURCE_PATH % share_access_rule_id
        return self._get(url, RESOURCE_NAME)

    @api_versions.wraps("2.45")
    def set_metadata(self, access, metadata):
        """Set or update metadata for share access rule.

        :param share_access_rule: either share access rule object or
            text with its ID.
        :param metadata: A list of keys to be set.
        """
        body = {'metadata': metadata}
        access_id = base.getid(access)
        url = RESOURCES_METADATA_PATH % access_id
        return self._update(url, body, "metadata")

    @api_versions.wraps("2.45")
    def unset_metadata(self, access, keys):
        """Unset metadata on a share access rule.

        :param keys: A list of keys on this object to be unset
        :return: None if successful, else API response on failure
        """
        for k in keys:
            url = RESOURCE_METADATA_PATH % (base.getid(access), k)
            self._delete(url)

    @api_versions.wraps("2.45")
    def access_list(self, share, search_opts=None):
        search_opts = search_opts or {}
        search_opts['share_id'] = base.getid(share)
        query_string = self._build_query_string(search_opts)
        url = RESOURCE_LIST_PATH + query_string
        return self._list(url, 'access_list')



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import os

from oslo_config import cfg
import oslo_log._options as log_options

from manilaclient import api_versions

# 1. Define opts

# "auth_opts" are used by functional tests that are located in
# directory "%project_root%/manilaclient/tests/functional"
auth_opts = [
    # Options for user with 'member' role.
    cfg.StrOpt("username",
               help="This should be the username of a user WITHOUT "
                    "administrative privileges."),
    cfg.StrOpt("tenant_name",
               help="The non-administrative user's tenant name."),
    cfg.StrOpt("password",
               secret=True,
               help="The non-administrative user's password."),
    cfg.StrOpt("auth_url",
               help="URL for where to find the OpenStack Identity public "
                    "API endpoint."),
    cfg.StrOpt("project_domain_name",
               help=("Project domain Name of user with 'member' role "
                     "as specified for auth v3.")),
    cfg.StrOpt("project_domain_id",
               help=("Project domain ID of user with 'member' role "
                     "as specified for auth v3.")),
    cfg.StrOpt("user_domain_name",
               help=("User domain Name of user with 'member' role "
                     "as specified for auth v3.")),
    cfg.StrOpt("user_domain_id",
               help=("User domain ID of user with 'member' role "
                     "as specified for auth v3.")),

    # Options for user with 'admin' role.
    cfg.StrOpt("admin_username",
               help="This should be the username of a user WITH "
                    "administrative privileges."),
    cfg.StrOpt("admin_tenant_name",
               help="The administrative user's tenant name."),
    cfg.StrOpt("admin_password",
               secret=True,
               help="The administrative user's password."),
    cfg.StrOpt("admin_auth_url",
               help="URL for where to find the OpenStack Identity admin "
                    "API endpoint."),
    cfg.StrOpt("admin_project_domain_name",
               help=("Project domain Name of user with 'admin' role "
                     "as specified for auth v3.")),
    cfg.StrOpt("admin_project_domain_id",
               help=("Project domain ID of user with 'admin' role "
                     "as specified for auth v3.")),
    cfg.StrOpt("admin_user_domain_name",
               help=("User domain Name of user with 'admin' role "
                     "as specified for auth v3.")),
    cfg.StrOpt("admin_user_domain_id",
               help=("User domain ID of user with 'admin' role "
                     "as specified for auth v3.")),

    # Other auth options
    cfg.BoolOpt("insecure",
                default=False,
                help="Disable SSL certificate verification."),
]

base_opts = [
    cfg.StrOpt("manila_exec_dir",
               default=os.environ.get(
                   'OS_MANILA_EXEC_DIR',
                   os.path.join(os.path.abspath('.'), '.tox/functional/bin')),
               help="The path to manilaclient to be executed."),
    cfg.BoolOpt("suppress_errors_in_cleanup",
                default=True,
                help="Whether to suppress errors with clean up operation "
                     "or not."),
]

share_opts = [
    cfg.StrOpt("min_api_microversion",
               default="1.0",
               help="The minimum API microversion is configured to be the "
                    "value of the minimum microversion supported by "
                    "Manilaclient functional tests. Defaults to 1.0."),
    cfg.StrOpt("max_api_microversion",
               default=api_versions.MAX_VERSION,
               help="The maximum API microversion is configured to be the "
                    "value of the latest microversion supported by "
                    "Manilaclient functional tests. Defaults to "
                    "manilaclient's max supported API microversion."),
    cfg.StrOpt("share_network",
               help="Share network Name or ID, that will be used for shares. "
                    "Some backend drivers require a share network for share "
                    "creation."),
    cfg.StrOpt("share_network_subnet",
               help="Share network subnet ID. Some backend drivers require a "
                    "share network for share creation."),
    cfg.StrOpt("admin_share_network",
               help="Share network Name or ID, that will be used for shares "
                    "in admin tenant."),
    cfg.StrOpt("share_type",
               help="Share type Name or ID, that will be used with share "
                    "creation scheduling. Optional."),
    cfg.ListOpt("enable_protocols",
                default=["nfs", "cifs"],
                help="List of all enabled protocols. The first protocol in "
                     "the list will be used as the default protocol."),
    cfg.IntOpt("build_interval",
               default=3,
               help="Time in seconds between share availability checks."),
    cfg.IntOpt("build_timeout",
               default=500,
               help="Timeout in seconds to wait for a share to become "
                    "available."),
    cfg.DictOpt('access_types_mapping',
                default={'nfs': 'ip', 'cifs': 'ip'},
                help="Dict contains access types mapping to share "
                     "protocol. It will be used to create access rules "
                     "for shares. Format: '<protocol>: <type1> <type2>',..."
                     "Allowed share protocols: nfs, cifs, cephfs, glusterfs, "
                     "hdfs."),
    cfg.DictOpt('access_levels_mapping',
                default={'nfs': 'rw ro', 'cifs': 'rw'},
                help="Dict contains access levels mapping to share "
                     "protocol. It will be used to create access rules for "
                     "shares. Format: '<protocol>: <level1> <level2>',... "
                     "Allowed share protocols: nfs, cifs, cephfs, glusterfs, "
                     "hdfs."),
    cfg.StrOpt("username_for_user_rules",
               default="stack",
               help="Username, that will be used in share access tests for "
                    "user type of access."),
    cfg.StrOpt("replication_type",
               default="readable",
               choices=["readable", "writable", "dr"],
               help="Replication type to be used when running replication "
                    "tests. This option is ignored if run_replication_tests "
                    "is set to False."),
    cfg.BoolOpt("run_replication_tests",
                default=True,
                help="Defines whether to run tests for share replication "
                     "or not. Disable this feature if manila driver used "
                     "doesn't support share replication."),
    cfg.BoolOpt("run_snapshot_tests",
                default=True,
                help="Defines whether to run tests that use share snapshots "
                     "or not. Disable this feature if used driver doesn't "
                     "support it."),
    cfg.BoolOpt("run_share_servers_tests",
                default=True,
                help="Defines whether to run tests that use share servers "
                     "or not. Disable this feature if used driver doesn't "
                     "support it or when autodeletion of share servers "
                     "is enabled."),
    cfg.BoolOpt("run_migration_tests",
                default=False,
                help="Defines whether to run share migration tests or not. "
                     "Disable this feature if there is no more than one "
                     "storage pool being tested or if used driver does not "
                     "support it."),
    cfg.BoolOpt("run_mount_snapshot_tests",
                default=False,
                help="Defines whether to run mountable snapshots tests or "
                     "not. Disable this feature if used driver doesn't "
                     "support it."),
    cfg.BoolOpt("run_manage_tests",
                default=False,
                help="Defines whether to run manage/unmanage tests or "
                     "not. Disable this feature if used driver does not "
                     "support it."),
    cfg.BoolOpt("run_share_servers_migration_tests",
                default=False,
                help="Defines whether to run share server migration tests or "
                     "not. Disable this feature if used driver does not "
                     "support it."),

]

# 2. Generate config

PROJECT_NAME = 'manilaclient'

DEFAULT_CONFIG_FILE = (
    os.environ.get('OS_%s_CONFIG_FILE' % PROJECT_NAME.upper()) or
    '%s.conf' % PROJECT_NAME)
DEFAULT_CONFIG_DIR = (
    os.environ.get('OS_%s_CONFIG_DIR' % PROJECT_NAME.upper()) or
    os.path.join(os.path.abspath(os.path.dirname(os.path.dirname(__file__))),
                 "etc/manilaclient")
)
DEFAULT_CONFIG_PATH = os.path.join(DEFAULT_CONFIG_DIR, DEFAULT_CONFIG_FILE)
FAILOVER_CONFIG_PATH = '/etc/%(pn)s/%(cn)s' % {
    'pn': PROJECT_NAME, 'cn': DEFAULT_CONFIG_FILE}
CONFIG_FILES = []

if os.path.isfile(DEFAULT_CONFIG_PATH):
    CONFIG_FILES.append(DEFAULT_CONFIG_PATH)
if os.path.isfile(FAILOVER_CONFIG_PATH):
    CONFIG_FILES.append(FAILOVER_CONFIG_PATH)

CONF = cfg.CONF

if CONFIG_FILES:
    CONF([], project=PROJECT_NAME, default_config_files=CONFIG_FILES)
else:
    CONF([], project=PROJECT_NAME)

# 3. Register opts

CONF.register_opts(auth_opts)
CONF.register_opts(base_opts)
CONF.register_opts(share_opts)

# 4. Define list_opts for config sample generator


def list_opts():
    """Return a list of oslo_config options available in Manilaclient."""
    opts = [
        (None, copy.deepcopy(auth_opts)),
        (None, copy.deepcopy(base_opts)),
        (None, copy.deepcopy(share_opts)),
    ]
    opts.extend(log_options.list_opts())
    return opts



# Copyright 2022 NetApp, Inc.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from manilaclient.tests.functional.osc import base


class ShareNetworkSubnetsCLITest(base.OSCClientTestBase):

    def test_openstack_share_network_create_check(self):
        share_network = self.create_share_network()

        check_result = self.check_create_network_subnet(
            share_network['id'])

        self.assertEqual('True', check_result['compatible'])
        self.assertEqual('{}', check_result['hosts_check_result'])

    def test_openstack_share_network_create_check_restart(self):
        share_network = self.create_share_network()

        check_result = self.check_create_network_subnet(
            share_network['id'], restart_check=True)

        self.assertEqual('True', check_result['compatible'])
        self.assertEqual('{}', check_result['hosts_check_result'])



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import io
import re
import sys
from unittest import mock

import ddt
import fixtures
from tempest.lib.cli import output_parser
from testtools import matchers

import manilaclient
from manilaclient.common import cliutils
from manilaclient.common import constants
from manilaclient import exceptions
from manilaclient import shell
from manilaclient.tests.unit import utils
from manilaclient.tests.unit.v2 import fakes


@ddt.ddt
class OpenstackManilaShellTest(utils.TestCase):

    FAKE_ENV = {
        'OS_USERNAME': 'username',
        'OS_PASSWORD': 'password',
        'OS_TENANT_NAME': 'tenant_name',
        'OS_AUTH_URL': 'http://no.where',
    }

    # Patch os.environ to avoid required auth info.
    def set_env_vars(self, env_vars):
        for k, v in env_vars.items():
            self.useFixture(fixtures.EnvironmentVariable(k, v))

    def shell_discover_client(self,
                              current_client,
                              os_api_version,
                              os_endpoint_type,
                              os_service_type,
                              client_args):
        return current_client, manilaclient.API_MAX_VERSION

    def shell(self, argstr):
        orig = sys.stdout
        try:
            sys.stdout = io.StringIO()
            _shell = shell.OpenStackManilaShell()
            _shell._discover_client = self.shell_discover_client
            _shell.main(argstr.split())
        except SystemExit:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            self.assertEqual(exc_value.code, 0)
        finally:
            out = sys.stdout.getvalue()
            sys.stdout.close()
            sys.stdout = orig

        return out

    @ddt.data(
        {},
        {'OS_AUTH_URL': 'http://foo.bar'},
        {'OS_AUTH_URL': 'http://foo.bar', 'OS_USERNAME': 'foo'},
        {'OS_AUTH_URL': 'http://foo.bar', 'OS_USERNAME': 'foo_user',
         'OS_PASSWORD': 'foo_password'},
        {'OS_TENANT_NAME': 'foo_tenant', 'OS_USERNAME': 'foo_user',
         'OS_PASSWORD': 'foo_password'},
        {'OS_TOKEN': 'foo_token'},
        {'OS_MANILA_BYPASS_URL': 'http://foo.foo'},
    )
    def test_main_failure(self, env_vars):
        self.set_env_vars(env_vars)
        with mock.patch.object(shell, 'client') as mock_client:
            self.assertRaises(exceptions.CommandError, self.shell, 'list')
            self.assertFalse(mock_client.Client.called)

    def test_main_success(self):
        env_vars = {
            'OS_AUTH_URL': 'http://foo.bar',
            'OS_USERNAME': 'foo_username',
            'OS_USER_ID': 'foo_user_id',
            'OS_PASSWORD': 'foo_password',
            'OS_TENANT_NAME': 'foo_tenant',
            'OS_TENANT_ID': 'foo_tenant_id',
            'OS_PROJECT_NAME': 'foo_project',
            'OS_PROJECT_ID': 'foo_project_id',
            'OS_PROJECT_DOMAIN_ID': 'foo_project_domain_id',
            'OS_PROJECT_DOMAIN_NAME': 'foo_project_domain_name',
            'OS_PROJECT_DOMAIN_ID': 'foo_project_domain_id',
            'OS_USER_DOMAIN_NAME': 'foo_user_domain_name',
            'OS_USER_DOMAIN_ID': 'foo_user_domain_id',
            'OS_CERT': 'foo_cert',
        }
        self.set_env_vars(env_vars)
        with mock.patch.object(shell, 'client') as mock_client:

            self.shell('list')

            mock_client.Client.assert_called_with(
                manilaclient.API_MAX_VERSION,
                username=env_vars['OS_USERNAME'],
                password=env_vars['OS_PASSWORD'],
                project_name=env_vars['OS_PROJECT_NAME'],
                auth_url=env_vars['OS_AUTH_URL'],
                insecure=False,
                region_name='',
                tenant_id=env_vars['OS_PROJECT_ID'],
                endpoint_type='publicURL',
                extensions=mock.ANY,
                service_type=constants.V2_SERVICE_TYPE,
                service_name='',
                retries=0,
                http_log_debug=False,
                cacert=None,
                use_keyring=False,
                force_new_token=False,
                user_id=env_vars['OS_USER_ID'],
                user_domain_id=env_vars['OS_USER_DOMAIN_ID'],
                user_domain_name=env_vars['OS_USER_DOMAIN_NAME'],
                project_domain_id=env_vars['OS_PROJECT_DOMAIN_ID'],
                project_domain_name=env_vars['OS_PROJECT_DOMAIN_NAME'],
                cert=env_vars['OS_CERT'],
                input_auth_token='',
                service_catalog_url='',
            )

    @ddt.data(
        {"env_vars": {"OS_MANILA_BYPASS_URL": "http://foo.url",
                      "OS_TOKEN": "foo_token"},
         "kwargs": {"--os-token": "bar_token",
                    "--bypass-url": "http://bar.url"},
         "expected": {"input_auth_token": "bar_token",
                      "service_catalog_url": "http://bar.url"}},
        {"env_vars": {"OS_MANILA_BYPASS_URL": "http://foo.url",
                      "OS_TOKEN": "foo_token"},
         "kwargs": {},
         "expected": {"input_auth_token": "foo_token",
                      "service_catalog_url": "http://foo.url"}},
        {"env_vars": {},
         "kwargs": {"--os-token": "bar_token",
                    "--bypass-url": "http://bar.url"},
         "expected": {"input_auth_token": "bar_token",
                      "service_catalog_url": "http://bar.url"}},
        {"env_vars": {"MANILACLIENT_BYPASS_URL": "http://foo.url",
                      "OS_TOKEN": "foo_token"},
         "kwargs": {},
         "expected": {"input_auth_token": "foo_token",
                      "service_catalog_url": "http://foo.url"}},
        {"env_vars": {"OS_TOKEN": "foo_token"},
         "kwargs": {"--bypass-url": "http://bar.url"},
         "expected": {"input_auth_token": "foo_token",
                      "service_catalog_url": "http://bar.url"}},
        {"env_vars": {"MANILACLIENT_BYPASS_URL": "http://foo.url",
                      "OS_MANILA_BYPASS_URL": "http://bar.url",
                      "OS_TOKEN": "foo_token"},
         "kwargs": {"--os-token": "bar_token"},
         "expected": {"input_auth_token": "bar_token",
                      "service_catalog_url": "http://bar.url"}},
    )
    @ddt.unpack
    def test_main_success_with_token(self, env_vars, kwargs, expected):
        self.set_env_vars(env_vars)
        with mock.patch.object(shell, "client") as mock_client:
            cmd = ""
            for k, v in kwargs.items():
                cmd += "%s=%s " % (k, v)
            cmd += "list"

            self.shell(cmd)

            mock_client.Client.assert_called_with(
                manilaclient.API_MAX_VERSION,
                username="",
                password="",
                project_name="",
                auth_url="",
                insecure=False,
                region_name="",
                tenant_id="",
                endpoint_type="publicURL",
                extensions=mock.ANY,
                service_type=constants.V2_SERVICE_TYPE,
                service_name="",
                retries=0,
                http_log_debug=False,
                cacert=None,
                use_keyring=False,
                force_new_token=False,
                user_id="",
                user_domain_id="",
                user_domain_name="",
                project_domain_id="",
                project_domain_name="",
                cert="",
                input_auth_token=expected["input_auth_token"],
                service_catalog_url=expected["service_catalog_url"],
            )

    @ddt.data(
        # default without any env var or kwargs
        {
            "env_vars": {"OS_TOKEN": "foo_token",
                         "OS_MANILA_BYPASS_URL": "http://bar.url"},
            "kwargs": {},
            "expected": {"input_auth_token": "foo_token",
                         "service_catalog_url": "http://bar.url",
                         "os_endpoint_type": "publicURL"}
        },
        # only env var
        {
            "env_vars": {"OS_TOKEN": "foo_token",
                         "OS_MANILA_BYPASS_URL": "http://bar.url",
                         "OS_MANILA_ENDPOINT_TYPE": "custom-endpoint-type"},
            "kwargs": {},
            "expected": {"input_auth_token": "foo_token",
                         "service_catalog_url": "http://bar.url",
                         "os_endpoint_type": "custom-endpoint-type"},
        },
        # only kwargs
        {
            "env_vars": {"OS_TOKEN": "foo_token",
                         "OS_MANILA_BYPASS_URL": "http://bar.url"},
            "kwargs": {"--endpoint-type": "custom-kwargs-endpoint-type"},
            "expected": {"input_auth_token": "foo_token",
                         "service_catalog_url": "http://bar.url",
                         "os_endpoint_type": "custom-kwargs-endpoint-type"},
        },
        # env var *and* kwargs (kwargs should win)
        {
            "env_vars": {"OS_TOKEN": "foo_token",
                         "OS_MANILA_BYPASS_URL": "http://bar.url",
                         "os_endpoint_type": "custom-env-endpoint-type"},
            "kwargs": {"--endpoint-type": "custom-kwargs-endpoint-type"},
            "expected": {"input_auth_token": "foo_token",
                         "service_catalog_url": "http://bar.url",
                         "os_endpoint_type": "custom-kwargs-endpoint-type"},
        }
    )
    @ddt.unpack
    def test_main_success_with_os_endpoint(self, env_vars, kwargs, expected):
        self.set_env_vars(env_vars)
        with mock.patch.object(shell, "client") as mock_client:
            cmd = ""
            for k, v in kwargs.items():
                cmd += "%s=%s " % (k, v)
            cmd += "list"

            self.shell(cmd)

            mock_client.Client.assert_called_with(
                manilaclient.API_MAX_VERSION,
                username="",
                password="",
                project_name="",
                auth_url="",
                insecure=False,
                region_name="",
                tenant_id="",
                endpoint_type=expected["os_endpoint_type"],
                extensions=mock.ANY,
                service_type=constants.V2_SERVICE_TYPE,
                service_name="",
                retries=0,
                http_log_debug=False,
                cacert=None,
                use_keyring=False,
                force_new_token=False,
                user_id="",
                user_domain_id="",
                user_domain_name="",
                project_domain_id="",
                project_domain_name="",
                cert="",
                input_auth_token=expected["input_auth_token"],
                service_catalog_url=expected["service_catalog_url"],
            )

    def test_help_unknown_command(self):
        self.assertRaises(exceptions.CommandError, self.shell, 'help foofoo')

    @ddt.data('list --help', '--help list', 'help list')
    def test_help_on_subcommand(self, cmd):
        required = [
            '.*?^usage: manila list',
            '.*?^List NAS shares with filters.',
        ]
        help_text = self.shell(cmd)
        for r in required:
            self.assertThat(help_text,
                            matchers.MatchesRegex(r, re.DOTALL | re.MULTILINE))

    def test_common_args_in_help_message(self):
        expected_args = (
            '--version', '', '--debug', '--os-cache', '--os-reset-cache',
            '--os-user-id', '--os-username', '--os-password',
            '--os-tenant-name', '--os-project-name', '--os-tenant-id',
            '--os-project-id', '--os-user-domain-id', '--os-user-domain-name',
            '--os-project-domain-id', '--os-project-domain-name',
            '--os-auth-url', '--os-region-name', '--service-type',
            '--service-name', '--share-service-name', '--endpoint-type',
            '--os-share-api-version', '--os-cacert', '--retries', '--os-cert',
        )

        help_text = self.shell('help')

        for expected_arg in expected_args:
            self.assertIn(expected_arg, help_text)


class CustomOpenStackManilaShell(shell.OpenStackManilaShell):

    @staticmethod
    @cliutils.arg(
        '--default-is-none',
        '--default_is_none',
        type=str,
        metavar='<redefined_metavar>',
        action='single_alias',
        help='Default value is None and metavar set.',
        default=None)
    def do_foo(cs, args):
        cliutils.print_dict({'key': args.default_is_none})

    @staticmethod
    @cliutils.arg(
        '--default-is-not-none',
        '--default_is_not_none',
        type=str,
        action='single_alias',
        help='Default value is not None and metavar not set.',
        default='bar')
    def do_bar(cs, args):
        cliutils.print_dict({'key': args.default_is_not_none})

    @staticmethod
    @cliutils.arg(
        '--list-like',
        '--list_like',
        nargs='*',
        action='single_alias',
        help='Default value is None, metavar not set and result is list.',
        default=None)
    def do_quuz(cs, args):
        cliutils.print_dict({'key': args.list_like})


@ddt.ddt
class AllowOnlyOneAliasAtATimeActionTest(utils.TestCase):
    FAKE_ENV = {
        'OS_USERNAME': 'username',
        'OS_PASSWORD': 'password',
        'OS_TENANT_NAME': 'tenant_name',
        'OS_AUTH_URL': 'http://no.where',
    }

    def setUp(self):
        super(self.__class__, self).setUp()
        for k, v in self.FAKE_ENV.items():
            self.useFixture(fixtures.EnvironmentVariable(k, v))
        self.mock_object(
            shell.client, 'get_client_class',
            mock.Mock(return_value=fakes.FakeClient))

    def shell_discover_client(self,
                              current_client,
                              os_api_version,
                              os_endpoint_type,
                              os_service_type,
                              client_args):
        return current_client, manilaclient.API_MAX_VERSION

    def shell(self, argstr):
        orig = sys.stdout
        try:
            sys.stdout = io.StringIO()
            _shell = CustomOpenStackManilaShell()
            _shell._discover_client = self.shell_discover_client
            _shell.main(argstr.split())
        except SystemExit:
            exc_type, exc_value, exc_traceback = sys.exc_info()
            self.assertEqual(exc_value.code, 0)
        finally:
            out = sys.stdout.getvalue()
            sys.stdout.close()
            sys.stdout = orig

        return out

    @ddt.data(
        ('--default-is-none foo', 'foo'),
        ('--default-is-none foo --default-is-none foo', 'foo'),
        ('--default-is-none foo --default_is_none foo', 'foo'),
        ('--default_is_none None', 'None'),
    )
    @ddt.unpack
    def test_foo_success(self, options_str, expected_result):
        output = self.shell('foo %s' % options_str)
        parsed_output = output_parser.details(output)
        self.assertEqual({'key': expected_result}, parsed_output)

    @ddt.data(
        '--default-is-none foo --default-is-none bar',
        '--default-is-none foo --default_is_none bar',
        '--default-is-none foo --default_is_none FOO',
    )
    def test_foo_error(self, options_str):
        self.assertRaises(
            matchers.MismatchError, self.shell, 'foo %s' % options_str)

    @ddt.data(
        ('--default-is-not-none bar', 'bar'),
        ('--default_is_not_none bar --default-is-not-none bar', 'bar'),
        ('--default_is_not_none bar --default_is_not_none bar', 'bar'),
        ('--default-is-not-none not_bar', 'not_bar'),
        ('--default_is_not_none None', 'None'),
    )
    @ddt.unpack
    def test_bar_success(self, options_str, expected_result):
        output = self.shell('bar %s' % options_str)
        parsed_output = output_parser.details(output)
        self.assertEqual({'key': expected_result}, parsed_output)

    @ddt.data(
        '--default-is-not-none foo --default-is-not-none bar',
        '--default-is-not-none foo --default_is_not_none bar',
        '--default-is-not-none bar --default_is_not_none BAR',
    )
    def test_bar_error(self, options_str):
        self.assertRaises(
            matchers.MismatchError, self.shell, 'bar %s' % options_str)

    @ddt.data(
        ('--list-like q=w', "['q=w']"),
        ('--list-like q=w --list_like q=w', "['q=w']"),
        ('--list-like q=w e=r t=y --list_like e=r t=y q=w',
         "['e=r', 'q=w', 't=y']"),
        ('--list_like q=w e=r t=y', "['e=r', 'q=w', 't=y']"),
    )
    @ddt.unpack
    def test_quuz_success(self, options_str, expected_result):
        output = self.shell('quuz %s' % options_str)
        parsed_output = output_parser.details(output)
        self.assertEqual({'key': expected_result}, parsed_output)

    @ddt.data(
        '--list-like q=w --list_like e=r t=y',
    )
    def test_quuz_error(self, options_str):
        self.assertRaises(
            matchers.MismatchError, self.shell, 'quuz %s' % options_str)



# Copyright (c) 2011 X.commerce, a business unit of eBay Inc.
# Copyright 2011 OpenStack Foundation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import manilaclient
from manilaclient import api_versions
from manilaclient.tests.unit.v2 import fake_clients as fakes
from manilaclient.v2 import client


class FakeClient(fakes.FakeClient):

    def __init__(self, *args, **kwargs):
        client.Client.__init__(
            self,
            manilaclient.API_MAX_VERSION,
            'username',
            'password',
            'project_id',
            'auth_url',
            input_auth_token='token',
            extensions=kwargs.get('extensions'),
            service_catalog_url='http://localhost:8786',
            api_version=kwargs.get("api_version", manilaclient.API_MAX_VERSION)
        )
        self.client = FakeHTTPClient(**kwargs)


fake_share_instance = {
    'id': 1234,
    'share_id': 'fake',
    'status': 'available',
    'availability_zone': 'fake',
    'share_network_id': 'fake',
    'share_server_id': 'fake',
}


def get_fake_export_location():
    return {
        'uuid': 'foo_el_uuid',
        'path': '/foo/el/path',
        'share_instance_id': 'foo_share_instance_id',
        'is_admin_only': False,
        'created_at': '2015-12-17T13:14:15Z',
        'updated_at': '2015-12-17T14:15:16Z',
    }


def get_fake_snapshot_export_location():
    return {
        'uuid': 'foo_el_uuid',
        'path': '/foo/el/path',
        'share_snapshot_instance_id': 'foo_share_instance_id',
        'is_admin_only': False,
        'created_at': '2017-01-17T13:14:15Z',
        'updated_at': '2017-01-17T14:15:16Z',
    }


class FakeHTTPClient(fakes.FakeHTTPClient):

    def get_(self, **kw):
        body = {
            "versions": [
                {
                    "status": "CURRENT",
                    "updated": "2015-07-30T11:33:21Z",
                    "links": [
                        {
                            "href": "http://docs.openstack.org/",
                            "type": "text/html",
                            "rel": "describedby",
                        },
                        {
                            "href": "http://localhost:8786/v2/",
                            "rel": "self",
                        }
                    ],
                    "min_version": "2.0",
                    "version": self.default_headers[
                        "X-Openstack-Manila-Api-Version"],
                    "id": "v2.0",
                }
            ]
        }
        return (200, {}, body)

    def get_availability_zones(self):
        availability_zones = {
            "availability_zones": [
                {"id": "368c5780-ad72-4bcf-a8b6-19e45f4fafoo",
                 "name": "foo",
                 "created_at": "2016-07-08T14:13:12.000000",
                 "updated_at": "2016-07-08T15:14:13.000000"},
                {"id": "368c5780-ad72-4bcf-a8b6-19e45f4fabar",
                 "name": "bar",
                 "created_at": "2016-07-08T14:13:12.000000",
                 "updated_at": "2016-07-08T15:14:13.000000"},
            ]
        }
        return (200, {}, availability_zones)

    def get_os_services(self, **kw):
        services = {
            "services": [
                {"status": "enabled",
                 "binary": "manila-scheduler",
                 "zone": "foozone",
                 "state": "up",
                 "updated_at": "2015-10-09T13:54:09.000000",
                 "host": "lucky-star",
                 "id": 1},
                {"status": "enabled",
                 "binary": "manila-share",
                 "zone": "foozone",
                 "state": "up",
                 "updated_at": "2015-10-09T13:54:05.000000",
                 "host": "lucky-star",
                 "id": 2},
            ]
        }
        return (200, {}, services)

    get_services = get_os_services

    def put_os_services_enable(self, **kw):
        return (200, {}, {'host': 'foo', 'binary': 'manila-share',
                          'disabled': False})

    put_services_enable = put_os_services_enable

    def put_os_services_disable(self, **kw):
        return (200, {}, {'host': 'foo', 'binary': 'manila-share',
                          'disabled': True})

    put_services_disable = put_os_services_disable

    def get_v2(self, **kw):
        body = {
            "versions": [
                {
                    "status": "CURRENT",
                    "updated": "2015-07-30T11:33:21Z",
                    "links": [
                        {
                            "href": "http://docs.openstack.org/",
                            "type": "text/html",
                            "rel": "describedby",
                        },
                        {
                            "href": "http://localhost:8786/v2/",
                            "rel": "self",
                        }
                    ],
                    "min_version": "2.0",
                    "version": "2.5",
                    "id": "v1.0",
                }
            ]
        }
        return (200, {}, body)

    def get_shares_1234(self, **kw):
        share = {
            'share': {
                'id': 1234,
                'name': 'sharename',
                'status': 'available',
            },
        }
        return (200, {}, share)

    def get_share_servers_1234(self, **kw):
        share_servers = {
            'share_server': {
                'id': 1234,
                'share_network_id': 'fake_network_id_1',
                'backend_details': {},
            },
        }
        return (200, {}, share_servers)

    def get_share_servers_5678(self, **kw):
        share_servers = {
            'share_server': {
                'id': 5678,
                'share_network_id': 'fake_network_id_2',
            },
        }
        return (200, {}, share_servers)

    def get_shares_1111(self, **kw):
        share = {'share': {'id': 1111, 'name': 'share1111'}}
        return (200, {}, share)

    def get_shares(self, **kw):
        endpoint = "http://127.0.0.1:8786/v2"
        share_id = '1234'
        shares = {
            'shares': [
                {
                    'id': share_id,
                    'name': 'sharename',
                    'links': [
                        {"href": endpoint + "/fake_project/shares/" + share_id,
                         "rel": "self"},
                    ],
                },
            ]
        }
        return (200, {}, shares)

    def get_shares_detail(self, **kw):
        endpoint = "http://127.0.0.1:8786/v2"
        share_id = '1234'

        shares = {
            'shares': [
                {
                    'id': share_id,
                    'name': 'sharename',
                    'status': 'fake_status',
                    'size': 1,
                    'host': 'fake_host',
                    'export_location': 'fake_export_location',
                    'snapshot_id': 'fake_snapshot_id',
                    'links': [
                        {"href": endpoint + "/fake_project/shares/" + share_id,
                         "rel": "self"},
                    ],
                },
            ],
        }

        if kw.get('with_count'):
            shares.update({'count': 2})

        return (200, {}, shares)

    def get_snapshots_1234(self, **kw):
        snapshot = {'snapshot': {'id': 1234, 'name': 'sharename'}}
        return (200, {}, snapshot)

    def get_share_servers(self, **kw):
        share_servers = {
            'share_servers': [
                {
                    'id': 1234,
                    'host': 'fake_host',
                    'status': 'fake_status',
                    'share_network': 'fake_share_nw',
                    'project_id': 'fake_project_id',
                    'updated_at': 'fake_updated_at',
                    'name': 'fake_name',
                    'share_name': 'fake_share_name',
                }
            ]
        }
        return (200, {}, share_servers)

    def post_snapshots_1234_action(self, body, **kw):
        _body = None
        resp = 202
        assert len(list(body)) == 1
        action = list(body)[0]
        if action in ('reset_status', 'os-reset_status'):
            assert 'status' in body.get(
                'reset_status', body.get('os-reset_status'))
        elif action in ('force_delete', 'os-force_delete'):
            assert body[action] is None
        elif action in ('unmanage', ):
            assert body[action] is None
        elif action in 'allow_access':
            assert 'access_type' in body['allow_access']
            assert 'access_to' in body['allow_access']
            _body = {'snapshot_access': body['allow_access']}
        elif action in 'deny_access':
            assert 'access_id' in body['deny_access']
        else:
            raise AssertionError("Unexpected action: %s" % action)
        return (resp, {}, _body)

    post_snapshots_5678_action = post_snapshots_1234_action

    def post_snapshots_manage(self, body, **kw):
        _body = {'snapshot': {'id': 'fake'}}
        resp = 202

        if not ('share_id' in body['snapshot']
                and 'provider_location' in body['snapshot']
                and 'driver_options' in body['snapshot']):
            resp = 422

        result = (resp, {}, _body)
        return result

    def _share_instances(self):
        instances = {
            'share_instances': [
                fake_share_instance
            ]
        }
        return (200, {}, instances)

    def put_quota_sets_1234(self, *args, **kwargs):
        return (200, {}, {})

    def get_quota_sets_1234(self, *args, **kwargs):
        quota_set = {
            'quota_set': {
                'id': '1234',
                'shares': 50,
                'gigabytes': 1000,
                'snapshots': 50,
                'snapshot_gigabytes': 1000,
                'share_networks': 10,
            }
        }
        return (200, {}, quota_set)

    def get_quota_sets_1234_detail(self, *args, **kwargs):
        quota_set = {
            'quota_set': {
                'id': '1234',
                'shares': {'in_use': 0,
                           'limit': 50,
                           'reserved': 0},
                'gigabytes': {'in_use': 0,
                              'limit': 10000,
                              'reserved': 0},
                'snapshots': {'in_use': 0,
                              'limit': 50,
                              'reserved': 0},
                'snapshot_gigabytes': {'in_use': 0,
                                       'limit': 1000,
                                       'reserved': 0},
                'share_networks': {'in_use': 0,
                                   'limit': 10,
                                   'reserved': 0},
            }
        }
        return (200, {}, quota_set)

    def get_share_instances(self, **kw):
        return self._share_instances()

    def get_share_instances_1234_export_locations(self, **kw):
        export_locations = {
            'export_locations': [
                get_fake_export_location(),
            ]
        }
        return (200, {}, export_locations)

    get_shares_1234_export_locations = (
        get_share_instances_1234_export_locations)

    def get_share_instances_1234_export_locations_fake_el_uuid(self, **kw):
        export_location = {'export_location': get_fake_export_location()}
        return (200, {}, export_location)

    get_shares_1234_export_locations_fake_el_uuid = (
        get_share_instances_1234_export_locations_fake_el_uuid)

    def get_shares_fake_instances(self, **kw):
        return self._share_instances()

    def get_shares_1234_instances(self, **kw):
        return self._share_instances()

    def get_share_instances_1234(self):
        return (200, {}, {'share_instance': fake_share_instance})

    def post_share_instances_1234_action(self, body, **kw):
        _body = None
        resp = 202
        assert len(list(body)) == 1
        action = list(body)[0]
        if action in ('reset_status', 'os-reset_status'):
            assert 'status' in body.get(
                'reset_status', body.get('os-reset_status'))
        elif action == 'os-force_delete':
            assert body[action] is None
        else:
            raise AssertionError("Unexpected share action: %s" % action)
        return (resp, {}, _body)

    def get_snapshots(self, **kw):
        snapshots = {
            'snapshots': [
                {
                    'id': 1234,
                    'status': 'available',
                    'name': 'sharename',
                }
            ]
        }
        return (200, {}, snapshots)

    def get_snapshots_detail(self, **kw):
        snapshots = {'snapshots': [{
            'id': 1234,
            'created_at': '2012-08-27T00:00:00.000000',
            'share_size': 1,
            'share_id': 4321,
            'status': 'available',
            'name': 'sharename',
            'display_description': 'description',
            'share_proto': 'type',
            'export_location': 'location',
        }]}
        return (200, {}, snapshots)

    def post_os_share_manage(self, body, **kw):
        _body = {'share': {'id': 'fake'}}
        resp = 202

        if not ('service_host' in body['share']
                and 'share_type' in body['share']
                and 'export_path' in body['share']
                and 'protocol' in body['share']
                and 'driver_options' in body['share']):
            resp = 422

        result = (resp, {}, _body)
        return result

    post_shares_manage = post_os_share_manage

    def post_share_servers_manage(self, body, **kw):
        _body = {'share_server': {'id': 'fake'}}
        resp = 202

        if not ('host' in body['share_server']
                and 'share_network' in body['share_server']
                and 'identifier' in body['share_server']):
            resp = 422

        result = (resp, {}, _body)
        return result

    def post_share_servers_1234_action(self, body, **kw):
        _body = None
        assert len(list(body)) == 1
        action = list(body)[0]

        if action in ('reset_status', ):
            assert 'status' in body.get(
                'reset_status', body.get('os-reset_status'))
            _body = {
                'reset_status': {'status': body['reset_status']['status']}
            }
        elif action in ('unmanage', ):
            assert 'force' in body[action]
        elif action in (
                'migration_cancel', 'migration_complete',
                'migration_get_progress'):
            assert body[action] is None
            if 'migration_get_progress' == action:
                _body = {'total_progress': 50,
                         'task_state': 'fake_task_state',
                         'destination_share_server_id': 'fake_dest_id'}
                return 200, {}, _body
            elif 'migration_complete' == action:
                _body = {'destination_share_server_id': 'fake_dest_id'}
                return 200, {}, _body
        elif action in (
                'migration_start', 'migration_check'):
            assert 'host' in body[action]
            if 'migration-check':
                _body = {
                    'compatible': True,
                    'capacity': True,
                    'capability': True,
                    'writable': True,
                    'nondisruptive': True,
                    'preserve_snapshots': True,
                }
                return 200, {}, _body
        elif action == 'reset_task_state':
            assert 'task_state' in body[action]

        resp = 202
        result = (resp, {}, _body)
        return result

    def post_os_share_unmanage_1234_unmanage(self, **kw):
        _body = None
        resp = 202
        result = (resp, {}, _body)
        return result

    def post_shares_1234_action(self, body, **kw):
        _body = None
        resp = 202
        assert len(list(body)) == 1
        action = list(body)[0]
        if action in ('os-allow_access', 'allow_access'):
            expected = ['access_to', 'access_type']
            actual = sorted(list(body[action]))
            err_msg = "expected '%s', actual is '%s'" % (expected, actual)
            assert expected == actual, err_msg
            _body = {'access': {}}
        elif action in ('os-deny_access', 'deny_access'):
            assert list(body[action]) == ['access_id']
        elif action in ('os-access_list', 'access_list'):
            assert body[action] is None
        elif action in ('os-reset_status', 'reset_status'):
            assert 'status' in body.get(
                'reset_status', body.get('os-reset_status'))
        elif action in ('os-force_delete', 'force_delete'):
            assert body[action] is None
        elif action in ('os-extend', 'os-shrink', 'extend', 'shrink'):
            assert body[action] is not None
            assert body[action]['new_size'] is not None
        elif action in ('unmanage', ):
            assert body[action] is None
        elif action in ('revert', ):
            assert body[action] is not None
            assert body[action]['snapshot_id'] is not None
        elif action in (
                'migration_cancel', 'migration_complete',
                'migration_get_progress'):
            assert body[action] is None
            if 'migration_get_progress' == action:
                _body = {'total_progress': 50}
                return 200, {}, _body
        elif action in (
                'os-migrate_share', 'migrate_share',
                'migration_start'):
            assert 'host' in body[action]
        elif action == 'reset_task_state':
            assert 'task_state' in body[action]
        elif action in ('soft_delete', 'restore'):
            assert body[action] is None
        else:
            raise AssertionError("Unexpected share action: %s" % action)
        return (resp, {}, _body)

    def post_shares_1111_action(self, body, **kw):
        _body = None
        resp = 202
        assert len(list(body)) == 1
        action = list(body)[0]
        if action in ('allow_access', 'os-allow_access'):
            expected = ['access_level', 'access_to', 'access_type']
            actual = sorted(list(body[action]))
            err_msg = "expected '%s', actual is '%s'" % (expected, actual)
            assert expected == actual, err_msg
            _body = {'access': {}}
        elif action in ('access_list', 'os-access_list'):
            assert body[action] is None
            _body = {
                'access_list': [{
                    'access_level': 'rw',
                    'state': 'active',
                    'id': '1122',
                    'access_type': 'ip',
                    'access_to': '10.0.0.7'
                }]
            }
        else:
            raise AssertionError("Unexpected share action: %s" % action)
        return (resp, {}, _body)

    def get_share_access_rules(self, **kw):
        access = {
            'access_list': [{
                'access_level': 'rw',
                'state': 'active',
                'id': '1122',
                'access_type': 'ip',
                'access_to': '10.0.0.7',
                'metadata': {'key1': 'v1'}
            }]
        }
        return (200, {}, access)

    def get_share_access_rules_9999(self, **kw):
        access = {
            'access': {
                'access_level': 'rw',
                'state': 'active',
                'id': '9999',
                'access_type': 'ip',
                'access_to': '10.0.0.7',
                'metadata': {'key1': 'v1'}
            }
        }
        return (200, {}, access)

    def put_share_access_rules_9999_metadata(self, **kw):
        return (200, {}, {'metadata': {'key1': 'v1', 'key2': 'v2'}})

    def delete_share_access_rules_9999_metadata_key1(self, **kw):
        return (200, {}, None)

    def get_shares_2222(self, **kw):
        share = {'share': {'id': 2222, 'name': 'sharename'}}
        return (200, {}, share)

    def post_shares_2222_action(self, body, **kw):
        return (202, {}, {'access': {}})

    def post_share_networks(self, **kwargs):
        return (202, {}, {'share_network': {}})

    def post_share_networks_1234_subnets(self, **kwargs):
        return (202, {}, {'share_network_subnet': {}})

    def post_shares(self, **kwargs):
        return (202, {}, {'share': {'id': '1234', 'status': 'creating'}})

    def post_snapshots(self, **kwargs):
        return (202, {}, {'snapshot': {}})

    def delete_shares_1234(self, **kw):
        return (202, {}, None)

    def delete_shares_share_abc(self, **kw):
        return (202, {}, None)

    def delete_shares_share_xyz(self, **kw):
        return (202, {}, None)

    def delete_snapshots_1234(self, **kwargs):
        return (202, {}, None)

    def delete_share_servers_1234(self, **kwargs):
        return (202, {}, None)

    def delete_share_servers_5678(self, **kwargs):
        return (202, {}, None)

    def delete_security_services_fake_security_service1(self, **kwargs):
        return (202, {}, None)

    def delete_security_services_fake_security_service2(self, **kwargs):
        return (202, {}, None)

    def delete_share_networks_fake_share_network1(self, **kwargs):
        return (202, {}, None)

    def delete_share_networks_fake_share_network2(self, **kwargs):
        return (202, {}, None)

    def delete_share_networks_1234_subnets_fake_subnet1(self, **kwargs):
        return (202, {}, None)

    def delete_share_networks_1234_subnets_fake_subnet2(self, **kwargs):
        return (202, {}, None)

    def delete_snapshots_fake_snapshot1(self, **kwargs):
        return (202, {}, None)

    def delete_snapshots_fake_snapshot2(self, **kwargs):
        return (202, {}, None)

    def post_snapshots_fake_snapshot_force1_action(self, **kwargs):
        return (202, {}, None)

    def post_snapshots_fake_snapshot_force2_action(self, **kwargs):
        return (202, {}, None)

    def delete_types_fake_type1(self, **kwargs):
        return (202, {}, None)

    def delete_types_fake_type2(self, **kwargs):
        return (202, {}, None)

    def delete_share_servers_fake_share_server1(self, **kwargs):
        return (202, {}, None)

    def delete_share_servers_fake_share_server2(self, **kwargs):
        return (202, {}, None)

    def put_share_networks_1111(self, **kwargs):
        share_network = {'share_network': {'id': 1111}}
        return (200, {}, share_network)

    def put_shares_1234(self, **kwargs):
        share = {'share': {'id': 1234, 'name': 'sharename'}}
        return (200, {}, share)

    def put_snapshots_1234(self, **kwargs):
        snapshot = {'snapshot': {'id': 1234, 'name': 'snapshot_name'}}
        return (200, {}, snapshot)

    def get_share_networks_1111(self, **kw):
        share_nw = {'share_network': {'id': 1111, 'name': 'fake_share_nw'}}
        return (200, {}, share_nw)

    def post_share_networks_1234_action(self, **kw):
        share_nw = {'share_network': {'id': 1111, 'name': 'fake_share_nw'}}
        return (200, {}, share_nw)

    def get_share_networks_detail(self, **kw):
        share_nw = {
            'share_networks': [
                {'id': 1234, 'name': 'fake_share_nw'},
                {'id': 4321, 'name': 'duplicated_name'},
                {'id': 4322, 'name': 'duplicated_name'},
            ]
        }
        return (200, {}, share_nw)

    def get_share_networks_1234_subnets_fake_subnet_id(self, **kw):
        subnet = {
            'share_network_subnet': {
                'id': 'fake_subnet_id',
            },
        }
        return (200, {}, subnet)

    def get_security_services(self, **kw):
        security_services = {
            'security_services': [
                {
                    'id': 1111,
                    'name': 'fake_security_service',
                    'type': 'fake_type',
                    'status': 'fake_status',
                },
            ],
        }
        return (200, {}, security_services)

    def get_security_services_detail(self, **kw):
        security_services = {
            'security_services': [
                {
                    'id': 1111,
                    'name': 'fake_security_service',
                    'description': 'fake_description',
                    'share_network_id': 'fake_share-network_id',
                    'user': 'fake_user',
                    'password': 'fake_password',
                    'domain': 'fake_domain',
                    'server': 'fake_server',
                    'dns_ip': 'fake_dns_ip',
                    'ou': 'fake_ou',
                    'type': 'fake_type',
                    'status': 'fake_status',
                    'project_id': 'fake_project_id',
                    'updated_at': 'fake_updated_at',
                },
            ],
        }
        return (200, {}, security_services)

    def get_security_services_1111(self, **kw):
        ss = {'security_service': {'id': 1111, 'name': 'fake_ss'}}
        return (200, {}, ss)

    def put_security_services_1111(self, **kwargs):
        ss = {'security_service': {'id': 1111, 'name': 'fake_ss'}}
        return (200, {}, ss)

    def get_scheduler_stats_pools(self, **kw):
        pools = {
            'pools': [
                {
                    'name': 'host1@backend1#pool1',
                    'host': 'host1',
                    'backend': 'backend1',
                    'pool': 'pool1',
                },
                {
                    'name': 'host1@backend1#pool2',
                    'host': 'host1',
                    'backend': 'backend1',
                    'pool': 'pool2',
                }
            ]
        }
        return (200, {}, pools)

    def get_scheduler_stats_pools_detail(self, **kw):
        pools = {
            'pools': [
                {
                    'name': 'host1@backend1#pool1',
                    'host': 'host1',
                    'backend': 'backend1',
                    'pool': 'pool1',
                    'capabilities': {'qos': True},
                },
                {
                    'name': 'host1@backend1#pool2',
                    'host': 'host1',
                    'backend': 'backend1',
                    'pool': 'pool2',
                    'capabilities': {'qos': False},
                }
            ]
        }
        return (200, {}, pools)

    fake_share_group = {
        'id': '1234',
        'availability_zone': 'nova',
        'share_network_id': None,
        'status': 'available',
        'name': 'share group name',
        'description': 'my share group',
    }

    def get_share_groups_detail(self, **kw):
        share_groups = {'share_groups': [self.fake_share_group]}
        return 200, {}, share_groups

    def get_share_groups_1234(self, **kw):
        share_group = {'share_group': self.fake_share_group}
        return 200, {}, share_group

    def put_share_groups_1234(self, **kwargs):
        share_group = {'share_group': self.fake_share_group}
        return 200, {}, share_group

    def delete_share_groups_1234(self, **kw):
        return 202, {}, None

    def post_share_groups_1234_action(self, **kw):
        return 202, {}, None

    def post_share_groups(self, body, **kw):
        share_group = {
            'share_group': {
                'id': 'fake-sg-id',
                'name': 'fake_name',
            }
        }
        return 202, {}, share_group

    fake_share_group_snapshot = {
        'id': '1234',
        'status': 'available',
        'name': 'share group name',
        'description': 'my share group',
    }

    def get_share_group_snapshots(self, **kw):
        sg_snapshots = {
            'share_group_snapshots': [self.fake_share_group_snapshot],
        }
        return 200, {}, sg_snapshots

    def get_share_group_snapshots_detail(self, **kw):
        sg_snapshots = {
            'share_group_snapshots': [self.fake_share_group_snapshot],
        }
        return 200, {}, sg_snapshots

    def get_share_group_snapshots_1234(self, **kw):
        sg_snapshot = {'share_group_snapshot': self.fake_share_group_snapshot}
        return 200, {}, sg_snapshot

    def put_share_group_snapshots_1234(self, **kwargs):
        sg_snapshot = {
            'share_group_snapshot': self.fake_share_group_snapshot,
        }
        return 200, {}, sg_snapshot

    def delete_share_group_snapshots_1234(self, **kw):
        return 202, {}, None

    def post_share_group_snapshots_1234_action(self, **kw):
        return 202, {}, None

    def post_share_group_snapshots(self, body, **kw):
        sg_snapshot = {
            'share_group_snapshot': {
                'id': 3,
                'name': 'cust_snapshot',
            }
        }
        return 202, {}, sg_snapshot

    fake_share_replica = {
        "id": "5678",
        "share_id": "1234",
        "availability_zone": "nova",
        "share_network_id": None,
        "export_locations": [],
        "share_server_id": None,
        "host": "",
        "status": "error",
        "replica_state": "error",
        "created_at": "2015-10-05T18:21:33.000000",
        "export_location": None,
    }

    def delete_share_replicas_1234(self, **kw):
        return (202, {}, None)

    def delete_share_replicas_fake_replica_0(self, **kw):
        return (202, {}, None)

    def delete_share_replicas_fake_replica_1(self, **kw):
        return (202, {}, None)

    def get_share_replicas_detail(self, **kw):
        replicas = {
            'share_replicas': [
                self.fake_share_replica,
            ]
        }
        return (200, {}, replicas)

    def get_share_replicas_5678(self, **kw):
        replicas = {'share_replica': self.fake_share_replica}
        return (200, {}, replicas)

    def get_share_replicas_5678_export_locations(self, **kw):
        export_locations = {
            'export_locations': [
                get_fake_export_location(),
            ]
        }
        return (200, {}, export_locations)

    def get_share_replicas_1234_export_locations(self, **kw):
        export_locations = {
            'export_locations': [
                get_fake_export_location(),
            ]
        }
        return (200, {}, export_locations)

    def get_share_replicas_1234_export_locations_fake_el_uuid(self, **kw):
        export_location = {'export_location': get_fake_export_location()}
        return (200, {}, export_location)

    def post_share_replicas(self, **kw):
        return (202, {}, {'share_replica': self.fake_share_replica})

    def post_share_replicas_1234_action(self, body, **kw):
        _body = None
        resp = 202
        assert len(list(body)) == 1
        action = list(body)[0]
        if action in ('reset_status', 'reset_replica_state'):
            attr = action.split('reset_')[1]
            assert attr in body.get(action)
        elif action in ('force_delete', 'resync', 'promote'):
            assert body[action] is None
        else:
            raise AssertionError("Unexpected share action: %s" % action)
        return (resp, {}, _body)

    #
    # Set/Unset metadata
    #
    def delete_shares_1234_metadata_test_key(self, **kw):
        return (204, {}, None)

    def delete_shares_1234_metadata_key1(self, **kw):
        return (204, {}, None)

    def delete_shares_1234_metadata_key2(self, **kw):
        return (204, {}, None)

    def post_shares_1234_metadata(self, **kw):
        return (204, {}, {'metadata': {'test_key': 'test_value'}})

    def put_shares_1234_metadata(self, **kw):
        return (200, {}, {"metadata": {"key1": "val1", "key2": "val2"}})

    def get_shares_1234_metadata(self, **kw):
        return (200, {}, {"metadata": {"key1": "val1", "key2": "val2"}})

    def get_types_default(self, **kw):
        return self.get_types_1(**kw)

    def get_types_1234(self, **kw):
        return (200, {}, {
            'share_type': {'id': 1234,
                           'name': 'test-type-1234',
                           'share_type_access:is_public': True,
                           'description': "test share type desc",
                           'extra_specs': {'test': 'test'},
                           'required_extra_specs': {'test': 'test'}}})

    def get_types(self, **kw):
        req_version = self.default_headers['X-Openstack-Manila-Api-Version']
        if not isinstance(req_version, api_versions.APIVersion):
            req_version = api_versions.APIVersion(req_version)
        response_body = {
            'share_types': [{'id': 1,
                             'name': 'test-type-1',
                             'extra_specs': {'test1': 'test1'},
                             'required_extra_specs': {'test': 'test'}},
                            {'id': 2,
                             'name': 'test-type-2',
                             'extra_specs': {'test1': 'test1'},
                             'required_extra_specs': {'test': 'test'}}]
        }

        if req_version >= api_versions.APIVersion('2.46'):
            response_body['share_types'][0]['is_default'] = False
            response_body['share_types'][1]['is_default'] = False

        return 200, {}, response_body

    def get_types_1(self, **kw):
        return (200, {}, {'share_type': {
            'id': 1,
            'name': 'test-type-1',
            'extra_specs': {'test': 'test'},
            'required_extra_specs': {'test': 'test'}}})

    def get_types_2(self, **kw):
        return (200, {}, {'share_type': {
            'id': 2,
            'name': 'test-type-2',
            'extra_specs': {'test': 'test'},
            'required_extra_specs': {'test': 'test'}}})

    def get_types_3(self, **kw):
        return (200, {}, {
            'share_type': {
                'id': 3,
                'name': 'test-type-3',
                'extra_specs': {},
                'os-share-type-access:is_public': False
            }
        })

    def get_types_4(self, **kw):
        return (200, {}, {
            'share_type': {
                'id': 4,
                'name': 'test-type-3',
                'extra_specs': {},
                'os-share-type-access:is_public': True
            }
        })

    def post_types(self, body, **kw):
        share_type = body['share_type']
        required_extra_specs = {
            "driver_handles_share_servers": share_type[
                'extra_specs']['driver_handles_share_servers'],
        }
        return (202, {}, {
            'share_type': {
                'id': 3,
                'name': 'test-type-3',
                'is_default': False,
                'description': 'test description',
                'extra_specs': share_type['extra_specs'],
                'required_extra_specs': required_extra_specs,
            }
        })

    def post_types_3_action(self, body, **kw):
        _body = None
        resp = 202
        assert len(list(body)) == 1
        action = list(body)[0]
        if action == 'addProjectAccess':
            assert 'project' in body['addProjectAccess']
        elif action == 'removeProjectAccess':
            assert 'project' in body['removeProjectAccess']
        else:
            raise AssertionError('Unexpected action: %s' % action)
        return (resp, {}, _body)

    def post_types_1_extra_specs(self, body, **kw):
        assert list(body) == ['extra_specs']
        return (200, {}, {'extra_specs': {'k': 'v'}})

    def delete_types_1_extra_specs_k(self, **kw):
        return(204, {}, None)

    def delete_types_1(self, **kw):
        return (202, {}, None)

    def get_types_3_os_share_type_access(self, **kw):
        return (200, {}, {'share_type_access': [
            {'share_type_id': '11111111-1111-1111-1111-111111111111',
             'project_id': '00000000-0000-0000-0000-000000000000'}
        ]})

    get_types_3_share_type_access = get_types_3_os_share_type_access

    fake_snapshot_instance = {
        "id": "1234",
        "snapshot_id": "5678",
        "status": "error",
    }

    def get_snapshot_instances(self, **kw):
        instances = {
            'snapshot_instances': [
                self.fake_snapshot_instance,
            ]
        }
        return (200, {}, instances)

    def get_snapshot_instances_detail(self, **kw):
        instances = {
            'snapshot_instances': [
                {
                    'id': '1234',
                    'snapshot_id': '5679',
                    'created_at': 'fake',
                    'updated_at': 'fake',
                    'status': 'fake',
                    'share_id': 'fake',
                    'share_instance_id': 'fake',
                    'progress': 'fake',
                    'provider_location': 'fake',
                }
            ]
        }
        return (200, {}, instances)

    def get_snapshot_instances_1234(self, **kw):
        instances = {'snapshot_instance': self.fake_snapshot_instance}
        return (200, {}, instances)

    def get_snapshot_instances_1234_export_locations_fake_el_id(self, **kw):
        return (200, {}, {'share_snapshot_export_location': {
            'id': 'fake_id', 'path': '/fake_path'}})

    def get_snapshots_1234_export_locations_fake_el_id(self, **kw):
        return (200, {}, {'share_snapshot_export_location': {
            'id': 'fake_id', 'path': '/fake_path'}})

    def get_snapshot_instances_1234_export_locations(
            self, **kw):
        snapshot_export_location = {'share_snapshot_export_locations':
                                    [get_fake_export_location()]}
        return (200, {}, snapshot_export_location)

    def get_snapshots_1234_export_locations(self):
        snapshot_export_location = {'share_snapshot_export_locations':
                                    [get_fake_export_location()]}
        return (200, {}, snapshot_export_location)

    def get_snapshots_1234_access_list(self, **kw):
        access_list = {'snapshot_access_list': [{
            'state': 'active',
            'id': '1234',
            'access_type': 'ip',
            'access_to': '6.6.6.6'
        }]}
        return (200, {}, access_list)

    def post_snapshot_instances_1234_action(self, body, **kw):
        _body = None
        resp = 202
        assert len(list(body)) == 1
        action = list(body)[0]
        if action == 'reset_status':
            assert 'status' in body.get(action)
        else:
            raise AssertionError("Unexpected share action: %s" % action)
        return (resp, {}, _body)

    def get_share_group_types_default(self, **kw):
        return self.get_share_group_types_1(**kw)

    def get_share_group_types(self, **kw):
        share_group_types = {
            'share_group_types': [
                {
                    'id': 1,
                    'name': 'test-group-type-1',
                    'group_specs': {
                        'key1': 'value1',
                    },
                    'share_types': [
                        'type1',
                        'type2',
                    ],
                    'is_public': True,
                }, {
                    'id': 2,
                    'name': 'test-type-2',
                    'group_specs': {
                        'key2': 'value2',
                    },
                    'share_types': [
                        'type3',
                        'type4',
                    ],
                    'is_public': False,
                },
            ],
        }

        req_version = self.default_headers['X-Openstack-Manila-Api-Version']
        if req_version >= api_versions.APIVersion('2.46'):
            share_group_types['share_group_types'][0]['is_default'] = False
            share_group_types['share_group_types'][1]['is_default'] = False

        return 200, {}, share_group_types

    def get_share_group_types_1(self, **kw):
        share_group_type = {
            'share_group_type': {
                'id': 1,
                'name': 'test-group-type-1',
                'group_specs': {
                    'key1': 'value1',
                },
                'share_types': [
                    'type1',
                    'type2',
                ],
                'is_public': True,
            },
        }
        return 200, {}, share_group_type

    def get_share_group_types_2(self, **kw):
        share_group_type = {
            'share_type': {
                'id': 2,
                'name': 'test-group-type-2',
                'group_specs': {
                    'key2': 'value2',
                },
                'share_types': [
                    'type3',
                    'type4',
                ],
                'is_public': True,
            },
        }
        return 200, {}, share_group_type

    def post_share_group_types(self, body, **kw):
        share_group_type = {
            'share_group_type': {
                'id': 1,
                'name': 'test-group-type-1',
                'share_types': body['share_group_type']['share_types'],
                'is_public': True,
            },
        }
        return 202, {}, share_group_type

    def post_share_group_types_1234_action(self, body, **kw):
        assert len(list(body)) == 1
        action = list(body)[0]
        if action == 'addProjectAccess':
            assert 'project' in body['addProjectAccess']
        elif action == 'removeProjectAccess':
            assert 'project' in body['removeProjectAccess']
        else:
            raise AssertionError('Unexpected action: %s' % action)
        return 202, {}, None

    def post_share_group_types_1_specs(self, body, **kw):
        assert list(body) == ['group_specs']
        return 200, {}, {'group_specs': {'k': 'v'}}

    def delete_share_group_types_1_specs_k(self, **kw):
        return 204, {}, None

    def delete_share_group_types_1234(self, **kw):
        return 202, {}, None

    def get_share_group_types_1234_access(self, **kw):
        sg_type_access = {
            'share_group_type_access': [{
                'group_type_id': '11111111-1111-1111-1111-111111111111',
                'project_id': '00000000-0000-0000-0000-000000000000',
            }],
        }
        return 200, {}, sg_type_access

    fake_message = {
        'id': 'fake message id',
        'action_id': '001',
        'detail_id': '002',
        'user_message': 'user message',
        'message_level': 'ERROR',
        'resource_type': 'SHARE',
        'resource_id': 'resource id',
        'created_at': '2015-08-27T09:49:58-05:00',
        'expires_at': '2015-09-27T09:49:58-05:00',
        'request_id': 'req-936666d2-4c8f-4e41-9ac9-237b43f8b848',
    }

    def get_messages(self, **kw):
        messages = {
            'messages': [self.fake_message],
        }
        return 200, {}, messages

    def get_messages_1234(self, **kw):
        message = {'message': self.fake_message}
        return 200, {}, message

    def delete_messages_1234(self, **kw):
        return 202, {}, None

    def delete_messages_5678(self, **kw):
        return 202, {}, None


def fake_create(url, body, response_key):
    return {'url': url, 'body': body, 'resp_key': response_key}


def fake_update(url, body, response_key):
    return {'url': url, 'body': body, 'resp_key': response_key}


class FakeQuotaSet(object):

    def __init__(self, dictionary):
        self.dictionary = dictionary

    def to_dict(self):
        return self.dictionary


class ShareNetwork(object):
    id = 'fake share network id'
    name = 'fake share network name'


class ShareType(object):
    id = 'fake share type id'
    name = 'fake share type name'


class ShareGroupType(object):
    id = 'fake group type id'
    name = 'fake group type name'
    share_types = [ShareType().id]
    is_public = False


class ShareGroupTypeAccess(object):
    id = 'fake group type access id'
    name = 'fake group type access name'


class ShareGroup(object):
    id = 'fake group id'
    share_types = [ShareType().id]
    group_type_id = ShareGroupType().id
    share_network_id = ShareNetwork().id
    name = 'fake name'
    description = 'fake description'
    availability_zone = 'fake az'


class ShareGroupSnapshot(object):
    id = 'fake group snapshot id'
    share_group_id = ShareGroup().id,
    share_network_id = ShareNetwork().id
    name = 'fake name'
    description = 'fake description'


class Message(object):
    id = 'fake message id'
    action_id = '001'
    detail_id = '002'
    user_message = 'user message'
    message_level = 'ERROR'
    resource_type = 'SHARE'
    resource_id = 'resource id'
    created_at = '2015-08-27T09:49:58-05:00'
    expires_at = '2015-09-27T09:49:58-05:00'
    request_id = 'req-936666d2-4c8f-4e41-9ac9-237b43f8b848'



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

from osc_lib.command import command
from osc_lib import exceptions
from osc_lib import utils

from manilaclient.common._i18n import _
from manilaclient.common import cliutils


class ListShareSnapshotInstance(command.Lister):
    """List all share snapshot instances."""
    _description = _("List all share snapshot instances")

    def get_parser(self, prog_name):
        parser = super(ListShareSnapshotInstance, self).get_parser(
            prog_name)
        parser.add_argument(
            "--snapshot",
            metavar="<snapshot>",
            default=None,
            help=_("Filter results by share snapshot ID.")
        )
        parser.add_argument(
            "--detailed",
            action="store_true",
            help=_("Show detailed information about snapshot instances. ")
        )

        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share
        snapshot = (share_client.share_snapshots.get(parsed_args.snapshot)
                    if parsed_args.snapshot else None)

        share_snapshot_instances = share_client.share_snapshot_instances.list(
            detailed=parsed_args.detailed,
            snapshot=snapshot,
        )

        list_of_keys = ['ID', 'Snapshot ID', 'Status']

        if (parsed_args.detailed):
            list_of_keys += ['Created At', 'Updated At', 'Share ID',
                             'Share Instance ID', 'Progress',
                             'Provider Location']

        return (list_of_keys, (utils.get_item_properties
                (s, list_of_keys) for s in share_snapshot_instances))


class ShowShareSnapshotInstance(command.ShowOne):
    """Show details about a share snapshot instance."""
    _description = _("Show details about a share snapshot instance.")

    def get_parser(self, prog_name):
        parser = super(ShowShareSnapshotInstance, self).get_parser(
            prog_name)
        parser.add_argument(
            "snapshot_instance",
            metavar="<snapshot_instance>",
            help=_("ID of the share snapshot instance.")
        )

        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share
        snapshot_instance = share_client.share_snapshot_instances.get(
            parsed_args.snapshot_instance)

        snapshot_instance_export_locations = (
            share_client.share_snapshot_instance_export_locations.list(
                snapshot_instance=snapshot_instance))

        snapshot_instance._info['export_locations'] = []
        for element_location in snapshot_instance_export_locations:
            element_location._info.pop('links', None)
            snapshot_instance._info['export_locations'].append(
                element_location._info)

        if parsed_args.formatter == 'table':
            snapshot_instance._info['export_locations'] = (
                cliutils.convert_dict_list_to_string(
                    snapshot_instance._info['export_locations']))

        snapshot_instance._info.pop('links', None)

        return self.dict2columns(snapshot_instance._info)


class SetShareSnapshotInstance(command.Command):
    """Explicitly update the state of a share snapshot instance."""
    _description = _("Explicitly update the state of a share snapshot "
                     "instance.")

    def get_parser(self, prog_name):
        parser = super(SetShareSnapshotInstance, self).get_parser(
            prog_name)
        parser.add_argument(
            "snapshot_instance",
            metavar="<snapshot_instance>",
            help=_("ID of the share snapshot instance to update.")
        )
        parser.add_argument(
            '--status',
            metavar='<status>',
            default='available',
            choices=['available', 'error', 'creating', 'deleting',
                     'error_deleting'],
            help=_('Indicate state to update the snapshot instance to. '
                   'Default is available.')
        )

        return parser

    def take_action(self, parsed_args):
        share_client = self.app.client_manager.share

        try:
            share_client.share_snapshot_instances.reset_state(
                parsed_args.snapshot_instance, parsed_args.status)
        except Exception as e:
            msg = _(
                "Failed to update share snapshot instance status: %s" % e)
            raise exceptions.CommandError(msg)



# Copyright (c) 2017 Hitachi Data Systems
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manilaclient import api_versions
from manilaclient import base


class ShareSnapshotExportLocation(base.Resource):
    """Represent an export location snapshot of a snapshot."""

    def __repr__(self):
        return "<ShareSnapshotExportLocation: %s>" % self.id

    def __getitem__(self, key):
        return self._info[key]


class ShareSnapshotExportLocationManager(base.ManagerWithFind):
    """Manage :class:`ShareSnapshotExportLocation` resources."""
    resource_class = ShareSnapshotExportLocation

    @api_versions.wraps("2.32")
    def list(self, snapshot=None, search_opts=None):
        return self._list("/snapshots/%s/export-locations" %
                          base.getid(snapshot),
                          'share_snapshot_export_locations')

    @api_versions.wraps("2.32")
    def get(self, export_location, snapshot=None):
        params = {
            "snapshot_id": base.getid(snapshot),
            "export_location_id": base.getid(export_location),
        }

        return self._get("/snapshots/%(snapshot_id)s/export-locations/"
                         "%(export_location_id)s" % params,
                         "share_snapshot_export_location")



# Copyright 2016 Huawei inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from manilaclient import api_versions
from manilaclient import base


class ShareSnapshotInstance(base.Resource):
    """A snapshot instance is an instance of a snapshot."""

    def __repr__(self):
        return "<SnapshotInstance: %s>" % self.id

    def reset_state(self, state):
        """Update snapshot instance's 'status' attr."""
        self.manager.reset_state(self, state)


class ShareSnapshotInstanceManager(base.ManagerWithFind):
    """Manage :class:`SnapshotInstances` resources."""
    resource_class = ShareSnapshotInstance

    @api_versions.wraps("2.19")
    def get(self, instance):
        """Get a snapshot instance.

        :param instance: either snapshot instance object or text with its ID.
        :rtype: :class:`ShareSnapshotInstance`
        """
        snapshot_instance_id = base.getid(instance)
        return self._get("/snapshot-instances/%s" % snapshot_instance_id,
                         "snapshot_instance")

    @api_versions.wraps("2.19")
    def list(self, detailed=False, snapshot=None, search_opts=None):
        """List all snapshot instances."""
        if detailed:
            url = '/snapshot-instances/detail'
        else:
            url = '/snapshot-instances'

        if snapshot:
            url += '?snapshot_id=%s' % base.getid(snapshot)
        return self._list(url, 'snapshot_instances')

    @api_versions.wraps("2.19")
    def reset_state(self, instance, state):
        """Reset the 'status' attr of the snapshot instance.

        :param instance: either snapshot instance object or its UUID.
        :param state: state to set the snapshot instance's 'status' attr to.
        """
        return self._action("reset_status", instance, {"status": state})

    def _action(self, action, instance, info=None, **kwargs):
        """Perform a snapshot instance 'action'."""
        body = {action: info}
        self.run_hooks('modify_body_for_action', body, **kwargs)
        url = ('/snapshot-instances/%s/action' %
               base.getid(instance))
        return self.api.client.post(url, body=body)
